@@ -27,10 +27,7 @@ if [[ ${UPSTREAM_DEV} ]]; then
         git+https://github.com/dask/distributed \
         git+https://github.com/dask/dask-expr \
         git+https://github.com/dask/fastparquet \
-        git+https://github.com/zarr-developers/zarr-python.git@main
-        # Zarr's default branch (`v3`) is still under development.
-        # Explicitly specify `main` until their default branch is ready.
-        # https://github.com/zarr-developers/zarr-python/issues/1922
+        git+https://github.com/zarr-developers/zarr-python
     mamba uninstall --force numpy pandas scipy numexpr numba sparse scikit-image h5py
     python -m pip install --no-deps --pre --retries 10 \
         -i https://pypi.anaconda.org/scientific-python-nightly-wheels/simple \
