@@ -1,8 +1,11 @@
+from __future__ import annotations
+
 from dask import config, datasets
 from dask._version import get_versions
 from dask.base import (
     annotate,
     compute,
+    get_annotations,
     is_dask_collection,
     optimize,
     persist,
