@@ -1,11 +1,17 @@
-import numpy as np
-from numpy.compat import basestring
+from __future__ import annotations
 
-from ..utils import derived_from
-from .core import asarray, blockwise, einsum_lookup
+from functools import reduce
+from operator import mul
+
+import numpy as np
+
+from dask.array._shuffle import _calculate_new_chunksizes
+from dask.array.core import asarray, blockwise, einsum_lookup
+from dask.utils import derived_from
 
 einsum_symbols = "abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
 einsum_symbols_set = set(einsum_symbols)
+from dask import config
 
 
 def chunk_einsum(*operands, **kwargs):
@@ -51,7 +57,7 @@ def parse_einsum_input(operands):
     if len(operands) == 0:
         raise ValueError("No input operands")
 
-    if isinstance(operands[0], basestring):
+    if isinstance(operands[0], str):
         subscripts = operands[0].replace(" ", "")
         operands = [asarray(o) for o in operands[1:]]
 
@@ -66,7 +72,7 @@ def parse_einsum_input(operands):
         tmp_operands = list(operands)
         operand_list = []
         subscript_list = []
-        for p in range(len(operands) // 2):
+        for _ in range(len(operands) // 2):
             operand_list.append(tmp_operands.pop(0))
             subscript_list.append(tmp_operands.pop(0))
 
@@ -199,7 +205,7 @@ def einsum(*operands, dtype=None, optimize=False, split_every=None, **kwargs):
 
     split_every: int >= 2 or dict(axis: int), optional
         Determines the depth of the recursive aggregation.
-        Deafults to ``None`` which would let dask heuristically
+        Defaults to ``None`` which would let dask heuristically
         decide a good default.
     """
 
@@ -228,6 +234,39 @@ def einsum(*operands, dtype=None, optimize=False, split_every=None, **kwargs):
     contract_inds = all_inds - set(outputs)
     ncontract_inds = len(contract_inds)
 
+    if len(inputs) > 1 and len(outputs) > 0:
+        # Calculate the increase in chunk size compared to the largest input chunk
+        max_chunk_sizes, max_chunk_size_input = [], 1
+        for op, input in zip(ops, inputs):
+            max_chunk_size_input = max(
+                reduce(mul, map(max, op.chunks)), max_chunk_size_input
+            )
+            max_chunk_sizes.extend(
+                max(op.chunks[i])
+                for i, inp in enumerate(input)
+                if inp not in contract_inds
+            )
+
+        max_chunk_size_output = reduce(mul, max_chunk_sizes)
+        factor = max_chunk_size_output / (
+            max_chunk_size_input * config.get("array.chunk-size-tolerance")
+        )
+
+        # Rechunk inputs to make input chunks smaller to avoid an increase in
+        # output chunks
+        new_ops = []
+        for op, input in zip(ops, inputs):
+            changeable_dimensions = {ctr for ctr, i in enumerate(input) if i in outputs}
+            f = max(factor ** (len(changeable_dimensions) / len(outputs)), 1)
+            result = _calculate_new_chunksizes(
+                op.chunks,
+                list(op.chunks),
+                changeable_dimensions,
+                reduce(mul, map(max, op.chunks)) / f,
+            )
+            new_ops.append(op.rechunk(result))
+        ops = new_ops
+
     # Introduce the contracted indices into the blockwise product
     # so that we get numpy arrays, not lists
     result = blockwise(
