@@ -3,6 +3,7 @@ from __future__ import annotations
 import itertools
 import os
 from collections.abc import Hashable, Iterable, Mapping, Sequence
+from dataclasses import fields, is_dataclass, replace
 from itertools import product
 from math import prod
 from typing import Any
@@ -10,18 +11,15 @@ from typing import Any
 import tlz as toolz
 
 import dask
+from dask import base, utils
+from dask._task_spec import Dict, GraphNode, List, Task, TaskRef, parse_input
 from dask.base import clone_key, get_name_from_key, tokenize
 from dask.core import flatten, ishashable, keys_in_tasks, reverse_dict
+from dask.delayed import Delayed, finalize
 from dask.highlevelgraph import HighLevelGraph, Layer
-from dask.optimization import SubgraphCallable, fuse
-from dask.typing import Graph, Key
-from dask.utils import (
-    _deprecated,
-    apply,
-    ensure_dict,
-    homogeneous_deepmap,
-    stringify_collection_keys,
-)
+from dask.optimization import fuse
+from dask.typing import Key
+from dask.utils import _deprecated, ensure_dict, homogeneous_deepmap
 
 
 class BlockwiseDep:
@@ -128,7 +126,7 @@ class BlockwiseDepDict(BlockwiseDep):
     >>> layer = Blockwise(
     ...     output="collection-name",
     ...     output_indices="i",
-    ...     dsk={"collection-name": (func, '_0')},
+    ...     task=Task("collection-name", func, TaskRef("_0")),
     ...     indices=[(dep, "i")],
     ...     numblocks={},
     ... )
@@ -165,7 +163,7 @@ class BlockwiseDepDict(BlockwiseDep):
             return self.mapping[idx]
         except KeyError as err:
             # If a DataFrame collection was converted
-            # to an Array collection, the dimesion of
+            # to an Array collection, the dimension of
             # `idx` may not agree with the keys in
             # `self.mapping`. In this case, we can
             # use `self.numblocks` to check for a key
@@ -242,17 +240,172 @@ def blockwise(
 ):
     """Create a Blockwise symbolic mutable mapping
 
-    This is like the ``make_blockwise_graph`` function, but rather than construct a
-    dict, it returns a symbolic Blockwise object.
+    Applies a function, ``func``, across blocks from many different input
+    collections.  We arrange the pattern with which those blocks interact with
+    sets of matching indices.  E.g.::
 
-    ``*arrind_pairs`` is similar to those in `make_blockwise_graph`, but in addition to
-    allowing for collections it can accept BlockwiseDep instances, which allows for lazy
-    evaluation of arguments to ``func`` which might be different for different
-    chunks/partitions.
+        blockwise(func, 'z', 'i', 'x', 'i', 'y', 'i')
+
+    yield an embarrassingly parallel communication pattern and is read as
+
+        $$ z_i = func(x_i, y_i) $$
+
+    More complex patterns may emerge, including multiple indices::
+
+        blockwise(func, 'z', 'ij', 'x', 'ij', 'y', 'ji')
+
+        $$ z_{ij} = func(x_{ij}, y_{ji}) $$
+
+    Indices missing in the output but present in the inputs results in many
+    inputs being sent to one function (see examples).
+
+    Examples
+    --------
+    Simple embarrassing map operation
+
+    >>> def inc(x):
+    ...    return x + 1
+    >>> dict(
+    ...     blockwise(
+    ...         inc,
+    ...         'z', 'ij',
+    ...         'x', 'ij',
+    ...         numblocks={'x': (2, 2)}
+    ...     )
+    ... )  # doctest: +NORMALIZE_WHITESPACE
+    {('z', 0, 0): <Task ('z', 0, 0) inc(Alias(('x', 0, 0)))>,
+    ('z', 0, 1): <Task ('z', 0, 1) inc(Alias(('x', 0, 1)))>,
+    ('z', 1, 0): <Task ('z', 1, 0) inc(Alias(('x', 1, 0)))>,
+    ('z', 1, 1): <Task ('z', 1, 1) inc(Alias(('x', 1, 1)))>}
+
+    Simple operation on two datasets x and y
+
+    >>> def add(x, y):
+    ...     return x + y
+    >>> dict(
+    ...     blockwise(
+    ...         add,
+    ...         'z', 'ij',
+    ...         'x', 'ij',
+    ...         'y', 'ij',
+    ...         numblocks={'x': (2, 2), 'y': (2, 2)}
+    ...     )
+    ... )  # doctest: +NORMALIZE_WHITESPACE
+    {('z', 0, 0): <Task ('z', 0, 0) add(Alias(('x', 0, 0)), Alias(('y', 0, 0)))>,
+    ('z', 0, 1): <Task ('z', 0, 1) add(Alias(('x', 0, 1)), Alias(('y', 0, 1)))>,
+    ('z', 1, 0): <Task ('z', 1, 0) add(Alias(('x', 1, 0)), Alias(('y', 1, 0)))>,
+    ('z', 1, 1): <Task ('z', 1, 1) add(Alias(('x', 1, 1)), Alias(('y', 1, 1)))>}
+
+    Operation that flips one of the datasets
+
+    >>> def addT(x, y):
+    ...     return x + y.T
+    >>> dict(
+    ...     blockwise(
+    ...         addT,
+    ...         'z', 'ij',
+    ...         'x', 'ij',
+    ...         'y', 'ji',
+    ...         numblocks={'x': (2, 2), 'y': (2, 2)}
+    ...     )
+    ... )  # doctest: +NORMALIZE_WHITESPACE
+    {('z', 0, 0): <Task ('z', 0, 0) addT(Alias(('x', 0, 0)), Alias(('y', 0, 0)))>,
+    ('z', 0, 1): <Task ('z', 0, 1) addT(Alias(('x', 0, 1)), Alias(('y', 1, 0)))>,
+    ('z', 1, 0): <Task ('z', 1, 0) addT(Alias(('x', 1, 0)), Alias(('y', 0, 1)))>,
+    ('z', 1, 1): <Task ('z', 1, 1) addT(Alias(('x', 1, 1)), Alias(('y', 1, 1)))>}
+
+    Dot product with contraction over ``j`` index.  Yields list arguments
+
+    >>> from dask.array.core import dotmany
+    >>> dict(
+    ...     blockwise(
+    ...         dotmany,
+    ...         'z', 'ik',
+    ...         'x', 'ij',
+    ...         'y', 'jk',
+    ...         numblocks={'x': (2, 2), 'y': (2, 2)}
+    ...     )
+    ... )  # doctest: +SKIP
+    {('z', 0,  0):
+        <Task ('z', 0, 0) dotmany(
+        List((Alias(('x', 0, 0)), Alias(('x', 0, 1)))),
+        List((Alias(('y', 0, 0)), Alias(('y', 1, 0)))))
+        >,
+    ('z', 0,  1):
+        <Task ('z', 0, 1) dotmany(
+        List((Alias(('x', 0, 0)), Alias(('x', 0, 1)))),
+        List((Alias(('y', 0, 1)), Alias(('y', 1, 1)))))
+        >,
+    ('z', 1,  0):
+        <Task ('z', 1, 0) dotmany(
+        List((Alias(('x', 1, 0)), Alias(('x', 1, 1)))),
+        List((Alias(('y', 0, 0)), Alias(('y', 1, 0)))))
+        >,
+    ('z', 1,  1):
+        <Task ('z', 1, 1) dotmany(
+        List((Alias(('x', 1, 0)), Alias(('x', 1, 1)))),
+        List((Alias(('y', 0, 1)), Alias(('y', 1, 1)))))
+        >
+    }
+
+    Pass ``concatenate=True`` to concatenate arrays ahead of time.
+    ``concatenate_axes`` is then called on the lists before the arguments are
+    passed to the function.
+
+    Supports Broadcasting rules
+
+    >>> dict(
+    ...     blockwise(
+    ...         add,
+    ...         'z', 'ij',
+    ...         'x', 'ij',
+    ...         'y', 'ij',
+    ...         concatenate=True,
+    ...         numblocks={'x': (1, 2), 'y': (2, 2)}
+    ...     )
+    ... )  # doctest: +SKIP
+    {
+    ('z', 0, 0): <Task ('z', 0, 0) add(Alias(('x', 0, 0)), Alias(('y', 0, 0)))>,
+    ('z', 0, 1): <Task ('z', 0, 1) add(Alias(('x', 0, 1)), Alias(('y', 0, 1)))>,
+    ('z', 1, 0): <Task ('z', 1, 0) add(Alias(('x', 0, 0)), Alias(('y', 1, 0)))>,
+    ('z', 1, 1): <Task ('z', 1, 1) add(Alias(('x', 0, 1)), Alias(('y', 1, 1)))>
+    }
+
+    Include literals by indexing with ``None``
+
+    >>> dict(
+    ...     blockwise(
+    ...         add,
+    ...         'z', 'i',
+    ...         'x', 'i',
+    ...         100, None,
+    ...         numblocks={'x': (2, )}
+    ...     )
+    ... )  # doctest: +NORMALIZE_WHITESPACE
+    {('z', 0): <Task ('z', 0) add(Alias(('x', 0)), DataNode(100))>,
+    ('z', 1): <Task ('z', 1) add(Alias(('x', 1)), DataNode(100))>}
+
+    If the broadcasted value is a delayed object or other dask collection, the
+    key has to be wrapped appropriately as an Alias object.
+
+    >>> from dask import delayed
+    >>> from dask._task_spec import Alias
+    >>> delayed_inc = delayed(inc)(5, dask_key_name='dinc')
+    >>> dict(
+    ...     blockwise(
+    ...         add,
+    ...         'z', 'i',
+    ...         'x', 'i',
+    ...         Alias(delayed_inc.key), None,
+    ...         numblocks={'x': (2, )}
+    ...     )
+    ... )  # doctest: +NORMALIZE_WHITESPACE
+    {('z', 0): <Task ('z', 0) add(Alias(('x', 0)), Alias('dinc'))>,
+    ('z', 1): <Task ('z', 1) add(Alias(('x', 1)), Alias('dinc'))>}
 
     See Also
     --------
-    make_blockwise_graph
+    dask.array.blockwise
     Blockwise
     """
     new_axes = new_axes or {}
@@ -284,6 +437,7 @@ def blockwise(
         inputs.append(name)
         inputs_indices.append(index)
 
+    task_args = list(map(TaskRef, map(blockwise_token, range(len(inputs)))))
     # Unpack delayed objects in kwargs
     new_keys = {n for c in dependencies for n in c.__dask_layers__()}
     if kwargs:
@@ -292,28 +446,15 @@ def blockwise(
             blockwise_token(i) for i in range(len(inputs), len(inputs) + len(new_keys))
         )
         sub = dict(zip(new_keys, new_tokens))
-        inputs.extend(new_keys)
+        inputs.extend(map(TaskRef, new_keys))
         inputs_indices.extend((None,) * len(new_keys))
-        kwargs = subs(kwargs, sub)
 
     indices = [(k, v) for k, v in zip(inputs, inputs_indices)]
-    keys = map(blockwise_token, range(len(inputs)))
-
-    # Construct local graph
-    if not kwargs:
-        subgraph = {output: (func,) + tuple(keys)}
-    else:
-        _keys = list(keys)
-        if new_keys:
-            _keys = _keys[: -len(new_keys)]
-        kwargs2 = (dict, list(map(list, kwargs.items())))
-        subgraph = {output: (apply, func, _keys, kwargs2)}
-
-    # Construct final output
+    task = Task(output, func, *task_args, **kwargs)
     subgraph = Blockwise(
         output,
         output_indices,
-        subgraph,
+        task,
         indices,
         numblocks=numblocks,
         concatenate=concatenate,
@@ -378,8 +519,8 @@ class Blockwise(Layer):
 
     output: str
     output_indices: tuple[str, ...]
-    dsk: Graph
-    indices: tuple[tuple[str, tuple[str, ...] | None], ...]
+    task: Task
+    indices: tuple[tuple[str | TaskRef, tuple[str, ...] | None], ...]
     numblocks: Mapping[str, Sequence[int]]
     concatenate: bool | None
     new_axes: Mapping[str, int]
@@ -390,8 +531,8 @@ class Blockwise(Layer):
         self,
         output: str,
         output_indices: Iterable[str],
-        dsk: Graph,
-        indices: Iterable[tuple[str | BlockwiseDep, Iterable[str] | None]],
+        task: Task,
+        indices: Iterable[tuple[str | TaskRef | BlockwiseDep, Iterable[str] | None]],
         numblocks: Mapping[str, Sequence[int]],
         concatenate: bool | None = None,
         new_axes: Mapping[str, int] | None = None,
@@ -403,23 +544,29 @@ class Blockwise(Layer):
         self.output = output
         self.output_indices = tuple(output_indices)
         self.output_blocks = output_blocks
-        self.dsk = dsk
+        self.task = task
+        assert isinstance(task, Task)
 
         # Remove `BlockwiseDep` arguments from input indices
         # and add them to `self.io_deps`.
         # TODO: Remove `io_deps` and handle indexable objects
         # in `self.indices` throughout `Blockwise`.
         _tmp_indices = []
+        numblocks = dict(numblocks)
+        io_deps = dict(io_deps or {})
         if indices:
-            numblocks = ensure_dict(numblocks, copy=True)
-            io_deps = ensure_dict(io_deps or {}, copy=True)
             for dep, ind in indices:
+                if ind is not None:
+                    # FIXME: The Blockwise API is a little weird this way
+                    assert not isinstance(
+                        dep, TaskRef
+                    ), "TaskRef objects are only allowed for broadcasted inputs with None as index."
                 if isinstance(dep, BlockwiseDep):
                     name = tokenize(dep)
                     io_deps[name] = dep
                     numblocks[name] = dep.numblocks
                 else:
-                    name = dep
+                    name = dep  # type: ignore
                 _tmp_indices.append((name, tuple(ind) if ind is not None else ind))
         self.numblocks = numblocks
         self.io_deps = io_deps or {}
@@ -456,11 +603,9 @@ class Blockwise(Layer):
             return self._cached_dict["dsk"]
         else:
             keys = tuple(map(blockwise_token, range(len(self.indices))))
-            dsk, _ = fuse(self.dsk, [self.output])
-            func = SubgraphCallable(dsk, self.output, keys)
 
-            dsk = make_blockwise_graph(
-                func,
+            dsk = _make_blockwise_graph(
+                self.task,
                 self.output,
                 self.output_indices,
                 *list(toolz.concat(self.indices)),
@@ -470,6 +615,7 @@ class Blockwise(Layer):
                 output_blocks=self.output_blocks,
                 dims=self.dims,
                 io_deps=self.io_deps,
+                keys=keys,
             )
 
             self._cached_dict = {"dsk": dsk}
@@ -519,7 +665,6 @@ class Blockwise(Layer):
         # Generate coordinate map
         (coord_maps, concat_axes, dummies) = _get_coord_mapping(
             self.dims,
-            self.output,
             self.output_indices,
             self.numblocks,
             self.indices,
@@ -529,6 +674,8 @@ class Blockwise(Layer):
         # Gather constant dependencies (for all output keys)
         const_deps = set()
         for arg, ind in self.indices:
+            if isinstance(arg, TaskRef):
+                arg = arg.key
             if ind is None:
                 try:
                     if arg in all_hlg_keys:
@@ -545,7 +692,7 @@ class Blockwise(Layer):
                 if ind is not None and arg not in self.io_deps:
                     arg_coords = tuple(coords[c] for c in cmap)
                     if axes:
-                        tups = lol_product((arg,), arg_coords)
+                        tups = _lol_product((arg,), arg_coords)
                         deps.update(flatten(tups))
                         if concatenate:
                             tups = (concatenate, tups, axes)
@@ -560,6 +707,8 @@ class Blockwise(Layer):
                 for out_coords in output_blocks:
                     key = (self.output,) + out_coords
                     valid_key_dep = io_dep[out_coords]
+                    if isinstance(valid_key_dep, TaskRef):
+                        valid_key_dep = valid_key_dep.key
                     key_deps[key] |= {valid_key_dep}
 
         return key_deps
@@ -568,7 +717,7 @@ class Blockwise(Layer):
         return Blockwise(
             self.output,
             self.output_indices,
-            self.dsk,
+            self.task,
             self.indices,
             self.numblocks,
             concatenate=self.concatenate,
@@ -617,15 +766,19 @@ class Blockwise(Layer):
         is_leaf = True
 
         indices = []
-        k: Key
+        k: Key | TaskRef
         for k, idxv in self.indices:
             # Note: k may not be a key and thus not be hashable in the case where
             # one or more args of blockwise() are sequences of literals;
             # e.g. k = (list, [0, 1, 2])
             # See https://github.com/dask/dask/issues/8978
+
             if ishashable(k) and k in names:
                 is_leaf = False
-                k = clone_key(k, seed)
+                k = clone_key(k, seed)  # type: ignore
+            elif isinstance(k, TaskRef) and k.key in names:
+                is_leaf = False
+                k = TaskRef(clone_key(k.key, seed))
 
             indices.append((k, idxv))
 
@@ -636,22 +789,27 @@ class Blockwise(Layer):
                 k = clone_key(k, seed)
             numblocks[k] = nbv
 
-        dsk = {clone_key(k, seed): v for k, v in self.dsk.items()}
-
         if bind_to is not None and is_leaf:
             from dask.graph_manipulation import chunks
 
             # It's always a Delayed generated by dask.graph_manipulation.checkpoint;
             # the layer name always matches the key
             assert isinstance(bind_to, str)
-            dsk = {k: (chunks.bind, v, f"_{len(indices)}") for k, v in dsk.items()}
-            indices.append((bind_to, None))
+            newtask = Task(
+                clone_key(self.task.key, seed),
+                chunks.bind,
+                self.task,
+                TaskRef(blockwise_token(len(indices))),
+            )
+            indices.append((TaskRef(bind_to), None))
+        else:
+            newtask = self.task.substitute({}, key=clone_key(self.task.key, seed))
 
         return (
             Blockwise(
                 output=clone_key(self.output, seed),
                 output_indices=self.output_indices,
-                dsk=dsk,
+                task=newtask,
                 indices=indices,
                 numblocks=numblocks,
                 concatenate=self.concatenate,
@@ -666,7 +824,6 @@ class Blockwise(Layer):
 
 def _get_coord_mapping(
     dims,
-    output,
     out_indices,
     numblocks,
     argpairs,
@@ -688,8 +845,6 @@ def _get_coord_mapping(
         Mapping between each index specified in `argpairs` and
         the number of output blocks for that index. Corresponds
         to the Blockwise `dims` attribute.
-    output : str
-        Corresponds to the Blockwise `output` attribute.
     out_indices : tuple
         Corresponds to the Blockwise `output_indices` attribute.
     numblocks : dict
@@ -707,7 +862,7 @@ def _get_coord_mapping(
             block_names.add(name)
             for x in ind:
                 all_indices.add(x)
-    assert set(numblocks) == block_names
+    assert set(numblocks) == block_names, (numblocks, block_names)
 
     dummy_indices = all_indices - set(out_indices)
 
@@ -758,8 +913,8 @@ def _get_coord_mapping(
     return coord_maps, concat_axes, dummies
 
 
-def make_blockwise_graph(
-    func,
+def _make_blockwise_graph(
+    task,
     output,
     out_indices,
     *arrind_pairs,
@@ -768,127 +923,15 @@ def make_blockwise_graph(
     new_axes=None,
     output_blocks=None,
     dims=None,
-    deserializing=False,
-    func_future_args=None,
-    return_key_deps=False,
     io_deps=None,
+    keys=None,
 ):
-    """Tensor operation
-
-    Applies a function, ``func``, across blocks from many different input
-    collections.  We arrange the pattern with which those blocks interact with
-    sets of matching indices.  E.g.::
-
-        make_blockwise_graph(func, 'z', 'i', 'x', 'i', 'y', 'i')
-
-    yield an embarrassingly parallel communication pattern and is read as
-
-        $$ z_i = func(x_i, y_i) $$
-
-    More complex patterns may emerge, including multiple indices::
-
-        make_blockwise_graph(func, 'z', 'ij', 'x', 'ij', 'y', 'ji')
-
-        $$ z_{ij} = func(x_{ij}, y_{ji}) $$
-
-    Indices missing in the output but present in the inputs results in many
-    inputs being sent to one function (see examples).
-
-    Examples
-    --------
-    Simple embarrassing map operation
-
-    >>> inc = lambda x: x + 1
-    >>> make_blockwise_graph(inc, 'z', 'ij', 'x', 'ij', numblocks={'x': (2, 2)})  # doctest: +SKIP
-    {('z', 0, 0): (inc, ('x', 0, 0)),
-     ('z', 0, 1): (inc, ('x', 0, 1)),
-     ('z', 1, 0): (inc, ('x', 1, 0)),
-     ('z', 1, 1): (inc, ('x', 1, 1))}
-
-    Simple operation on two datasets
-
-    >>> add = lambda x, y: x + y
-    >>> make_blockwise_graph(add, 'z', 'ij', 'x', 'ij', 'y', 'ij', numblocks={'x': (2, 2),
-    ...                                                      'y': (2, 2)})  # doctest: +SKIP
-    {('z', 0, 0): (add, ('x', 0, 0), ('y', 0, 0)),
-     ('z', 0, 1): (add, ('x', 0, 1), ('y', 0, 1)),
-     ('z', 1, 0): (add, ('x', 1, 0), ('y', 1, 0)),
-     ('z', 1, 1): (add, ('x', 1, 1), ('y', 1, 1))}
-
-    Operation that flips one of the datasets
-
-    >>> addT = lambda x, y: x + y.T  # Transpose each chunk
-    >>> #                                        z_ij ~ x_ij y_ji
-    >>> #               ..         ..         .. notice swap
-    >>> make_blockwise_graph(addT, 'z', 'ij', 'x', 'ij', 'y', 'ji', numblocks={'x': (2, 2),
-    ...                                                       'y': (2, 2)})  # doctest: +SKIP
-    {('z', 0, 0): (add, ('x', 0, 0), ('y', 0, 0)),
-     ('z', 0, 1): (add, ('x', 0, 1), ('y', 1, 0)),
-     ('z', 1, 0): (add, ('x', 1, 0), ('y', 0, 1)),
-     ('z', 1, 1): (add, ('x', 1, 1), ('y', 1, 1))}
-
-    Dot product with contraction over ``j`` index.  Yields list arguments
-
-    >>> make_blockwise_graph(dotmany, 'z', 'ik', 'x', 'ij', 'y', 'jk', numblocks={'x': (2, 2),
-    ...                                                          'y': (2, 2)})  # doctest: +SKIP
-    {('z', 0, 0): (dotmany, [('x', 0, 0), ('x', 0, 1)],
-                            [('y', 0, 0), ('y', 1, 0)]),
-     ('z', 0, 1): (dotmany, [('x', 0, 0), ('x', 0, 1)],
-                            [('y', 0, 1), ('y', 1, 1)]),
-     ('z', 1, 0): (dotmany, [('x', 1, 0), ('x', 1, 1)],
-                            [('y', 0, 0), ('y', 1, 0)]),
-     ('z', 1, 1): (dotmany, [('x', 1, 0), ('x', 1, 1)],
-                            [('y', 0, 1), ('y', 1, 1)])}
-
-    Pass ``concatenate=True`` to concatenate arrays ahead of time
-
-    >>> make_blockwise_graph(f, 'z', 'i', 'x', 'ij', 'y', 'ij', concatenate=True,
-    ...     numblocks={'x': (2, 2), 'y': (2, 2,)})  # doctest: +SKIP
-    {('z', 0): (f, (concatenate_axes, [('x', 0, 0), ('x', 0, 1)], (1,)),
-                   (concatenate_axes, [('y', 0, 0), ('y', 0, 1)], (1,)))
-     ('z', 1): (f, (concatenate_axes, [('x', 1, 0), ('x', 1, 1)], (1,)),
-                   (concatenate_axes, [('y', 1, 0), ('y', 1, 1)], (1,)))}
-
-    Supports Broadcasting rules
-
-    >>> make_blockwise_graph(add, 'z', 'ij', 'x', 'ij', 'y', 'ij', numblocks={'x': (1, 2),
-    ...                                                      'y': (2, 2)})  # doctest: +SKIP
-    {('z', 0, 0): (add, ('x', 0, 0), ('y', 0, 0)),
-     ('z', 0, 1): (add, ('x', 0, 1), ('y', 0, 1)),
-     ('z', 1, 0): (add, ('x', 0, 0), ('y', 1, 0)),
-     ('z', 1, 1): (add, ('x', 0, 1), ('y', 1, 1))}
-
-    Support keyword arguments with apply
-
-    >>> def f(a, b=0): return a + b
-    >>> make_blockwise_graph(f, 'z', 'i', 'x', 'i', numblocks={'x': (2,)}, b=10)  # doctest: +SKIP
-    {('z', 0): (apply, f, [('x', 0)], {'b': 10}),
-     ('z', 1): (apply, f, [('x', 1)], {'b': 10})}
-
-    Include literals by indexing with ``None``
-
-    >>> make_blockwise_graph(add, 'z', 'i', 'x', 'i', 100, None,  numblocks={'x': (2,)})  # doctest: +SKIP
-    {('z', 0): (add, ('x', 0), 100),
-     ('z', 1): (add, ('x', 1), 100)}
-
-    See Also
-    --------
-    dask.array.blockwise
-    dask.blockwise.blockwise
-    """
-
     if numblocks is None:
         raise ValueError("Missing required numblocks argument.")
     new_axes = new_axes or {}
     io_deps = io_deps or {}
     argpairs = list(toolz.partition(2, arrind_pairs))
 
-    if return_key_deps:
-        key_deps = {}
-
-    if deserializing:
-        from distributed.protocol.serialize import to_serialize
-
     if concatenate is True:
         from dask.array.core import concatenate_axes as concatenate
 
@@ -899,7 +942,6 @@ def make_blockwise_graph(
     # the actual graph
     (coord_maps, concat_axes, dummies) = _get_coord_mapping(
         dims,
-        output,
         out_indices,
         numblocks,
         argpairs,
@@ -915,79 +957,48 @@ def make_blockwise_graph(
     output_blocks = output_blocks or list(
         itertools.product(*[range(dims[i]) for i in out_indices])
     )
+    from dask._task_spec import DataNode
 
     dsk = {}
-    # Create argument lists
     for out_coords in output_blocks:
-        deps = set()
+        this_task = task
         coords = out_coords + dummies
         args = []
-        for cmap, axes, (arg, ind) in zip(coord_maps, concat_axes, argpairs):
+        for cmap, axes, (arg, ind), key in zip(
+            coord_maps, concat_axes, argpairs, keys, strict=True
+        ):
+            if key not in task.dependencies:
+                # FIXME: This feels like a bug
+                continue
             if ind is None:
-                if deserializing:
-                    args.append(stringify_collection_keys(arg))
+                if not isinstance(arg, (GraphNode, TaskRef)):
+                    this_task = this_task.substitute({key: DataNode(None, arg)})
                 else:
-                    args.append(arg)
+                    this_task = this_task.substitute({key: arg})
+                continue
+            arg_coords = tuple(coords[c] for c in cmap)
+            if arg in io_deps:
+                val = parse_input(io_deps[arg].get(arg_coords, arg_coords))
+                if not isinstance(val, GraphNode):
+                    val = DataNode(None, val)
+                this_task = this_task.substitute({key: val})
             else:
-                arg_coords = tuple(coords[c] for c in cmap)
+                subs = {}
                 if axes:
-                    tups = lol_product((arg,), arg_coords)
-                    if arg not in io_deps:
-                        deps.update(flatten(tups))
-
+                    tups = _lol_product((arg,), arg_coords, as_taskref=True)
                     if concatenate:
-                        tups = (concatenate, tups, axes)
+                        tups = Task(key, concatenate, tups, axes)
+                    subs[key] = tups
                 else:
-                    tups = (arg,) + arg_coords
-                    if arg not in io_deps:
-                        deps.add(tups)
-                # Replace "place-holder" IO keys with "real" args
-                if arg in io_deps:
-                    # We don't want to stringify keys for args
-                    # we are replacing here
-                    idx = tups[1:]
-                    args.append(io_deps[arg].get(idx, idx))
-                elif deserializing:
-                    args.append(stringify_collection_keys(tups))
-                else:
-                    args.append(tups)
-        out_key = (output,) + out_coords
-
-        if deserializing:
-            deps.update(func_future_args)
-            args += list(func_future_args)
-
-        # Construct a function/args/kwargs dict if we
-        # do not have a nested task (i.e. concatenate=False).
-        # TODO: Avoid using the iterate_collection-version
-        # of to_serialize if we know that are no embedded
-        # Serialized/Serialize objects in args and/or kwargs.
-        if deserializing and isinstance(func, bytes):
-            dsk[out_key] = {"function": func, "args": to_serialize(args)}
-        else:
-            args.insert(0, func)
-            val = tuple(args)
-            # May still need to serialize (if concatenate=True)
-            dsk[out_key] = to_serialize(val) if deserializing else val
-
-        if return_key_deps:
-            key_deps[out_key] = deps
-
-    if return_key_deps:
-        # Add valid-key dependencies from io_deps
-        for key, io_dep in io_deps.items():
-            if io_dep.produces_keys:
-                for out_coords in output_blocks:
-                    key = (output,) + out_coords
-                    valid_key_dep = io_dep[out_coords]
-                    key_deps[key] |= {valid_key_dep}
-
-        return dsk, key_deps
-    else:
-        return dsk
+                    subs[key] = (arg, *arg_coords)
+                this_task = this_task.substitute(subs)
+        new_key = (output,) + out_coords
+        assert isinstance(this_task, Task)
+        dsk[new_key] = Task.fuse(this_task, *args, key=new_key)
+    return dsk
 
 
-def lol_product(head, values):
+def _lol_product(head, values, as_taskref=False):
     """List of list of tuple keys, similar to `itertools.product`.
 
     Parameters
@@ -1000,18 +1011,32 @@ def lol_product(head, values):
 
     Examples
     --------
-    >>> lol_product(('x',), (1, 2, 3))
+    >>> _lol_product(('x',), (1, 2, 3))
     ('x', 1, 2, 3)
-    >>> lol_product(('x',), (1, [2, 3], 4, [5, 6]))  # doctest: +NORMALIZE_WHITESPACE
+    >>> _lol_product(('x',), (1, [2, 3], 4, [5, 6]))  # doctest: +NORMALIZE_WHITESPACE
     [[('x', 1, 2, 4, 5), ('x', 1, 2, 4, 6)],
      [('x', 1, 3, 4, 5), ('x', 1, 3, 4, 6)]]
     """
     if not values:
+        if as_taskref:
+            return TaskRef(head)
         return head
     elif isinstance(values[0], list):
-        return [lol_product(head + (x,), values[1:]) for x in values[0]]
+        # FIXME: Constructor of List is odd
+        if as_taskref:
+            return List(
+                *(
+                    _lol_product(head + (x,), values[1:], as_taskref=as_taskref)
+                    for x in values[0]
+                )
+            )
+        else:
+            return list(
+                _lol_product(head + (x,), values[1:], as_taskref=as_taskref)
+                for x in values[0]
+            )
     else:
-        return lol_product(head + (values[0],), values[1:])
+        return _lol_product(head + (values[0],), values[1:], as_taskref=as_taskref)
 
 
 def lol_tuples(head, ind, values, dummies):
@@ -1138,16 +1163,14 @@ def _optimize_blockwise(full_graph, keys=()):
                 blockwise_layers.add(dep)
 
                 # traverse further to this child's children
+                output_indices = set(layers[dep].output_indices)
+                input_indices = {
+                    i for _, ind in layers[dep].indices if ind for i in ind
+                }
+                is_io_superset = output_indices.issuperset(input_indices)
                 for d in full_graph.dependencies.get(dep, ()):
                     # Don't allow reductions to proceed
-                    output_indices = set(layers[dep].output_indices)
-                    input_indices = {
-                        i for _, ind in layers[dep].indices if ind for i in ind
-                    }
-
-                    if len(dependents[d]) <= 1 and output_indices.issuperset(
-                        input_indices
-                    ):
+                    if is_io_superset and len(dependents[d]) <= 1:
                         deps.add(d)
                     else:
                         stack.append(d)
@@ -1286,13 +1309,14 @@ def rewrite_blockwise(inputs):
     indices = list(inputs[root].indices)
     new_axes = inputs[root].new_axes
     concatenate = inputs[root].concatenate
-    dsk = dict(inputs[root].dsk)
+    task = inputs[root].task
+    dsk = {task.key: task}
 
     changed = True
     while changed:
         changed = False
-        for i, (dep, ind) in enumerate(indices):
-            if ind is None:
+        for i, (dep, current_dep_indices) in enumerate(indices):
+            if current_dep_indices is None:
                 continue
             if dep not in inputs:
                 continue
@@ -1303,20 +1327,23 @@ def rewrite_blockwise(inputs):
             # (in different iteration orders) into a single
             # subgraph key/dependency
             # (see: https://github.com/dask/dask/issues/8535)
-            local_dep = dep if dep == root else _unique_dep(dep, ind)
+            local_dep = dep if dep == root else _unique_dep(dep, current_dep_indices)
 
             # Replace _n with dep name in existing tasks
             # (inc, _0) -> (inc, 'b')
-            dsk = {k: subs(v, {blockwise_token(i): local_dep}) for k, v in dsk.items()}
+            dsk = {
+                k: v.substitute({blockwise_token(i): local_dep}) for k, v in dsk.items()
+            }
 
             # Remove current input from input indices
             # [('a', 'i'), ('b', 'i')] -> [('a', 'i')]
-            _, current_dep_indices = indices.pop(i)
+            indices.pop(i)
+
             sub = {
                 blockwise_token(i): blockwise_token(i - 1)
                 for i in range(i + 1, len(indices) + 1)
             }
-            dsk = subs(dsk, sub)
+            dsk = {k: v.substitute(sub) for k, v in dsk.items()}
 
             # Change new input_indices to match give index from current computation
             # [('c', j')] -> [('c', 'i')]
@@ -1349,11 +1376,11 @@ def rewrite_blockwise(inputs):
                     index_map[id_key] = len(indices)
                     sub[blockwise_token(ii)] = blockwise_token(len(indices))
                     indices.append(index)
-            new_dsk = subs(inputs[dep].dsk, sub)
-
-            # Change new_dsk key to match local_dep
-            if dep != local_dep and dep in new_dsk:
-                new_dsk[local_dep] = new_dsk.pop(dep)
+            if dep != local_dep:
+                key = local_dep
+            else:
+                key = dep
+            new_dsk = {key: inputs[dep].task.substitute(sub, key=key)}
 
             # indices.extend(new_indices)
             dsk.update(new_dsk)
@@ -1373,8 +1400,8 @@ def rewrite_blockwise(inputs):
             new_indices.append(x)
 
     sub = {blockwise_token(k): blockwise_token(v) for k, v in sub.items()}
-    dsk = {k: subs(v, sub) for k, v in dsk.items() if k not in sub.keys()}
-
+    dsk = {k: v.substitute(sub) for k, v in dsk.items() if k not in sub.keys()}
+    task = Task.fuse(*dsk.values(), key=root)
     indices_check = {k for k, v in indices if v is not None}
     numblocks = toolz.merge([inp.numblocks for inp in inputs.values()])
     numblocks = {k: v for k, v in numblocks.items() if v is None or k in indices_check}
@@ -1387,7 +1414,7 @@ def rewrite_blockwise(inputs):
     return Blockwise(
         root,
         inputs[root].output_indices,
-        dsk,
+        task,
         new_indices,
         numblocks=numblocks,
         new_axes=new_axes,
@@ -1545,3 +1572,97 @@ def fuse_roots(graph: HighLevelGraph, keys: list):
             dependencies[name] = set()
 
     return HighLevelGraph(layers, dependencies)
+
+
+def _blockwise_unpack_collections_task_spec(expr):
+    # FIXME This is a copy of the delayed.unpack_collections function with the
+    # addition of the TaskSpec class. Eventually this should all be consolidated
+    # but to reduce the number of changes we'll vendor this here
+    # FIXME: There is also a dask.base version of unpack_collections that looks
+    # similar but is different. At the very least the names should be fixed
+    if isinstance(expr, Delayed):
+        return TaskRef(expr._key), (expr,)
+
+    if base.is_dask_collection(expr):
+        if hasattr(expr, "optimize"):
+            # Optimize dask-expr collections
+            expr = expr.optimize()
+
+        finalized = finalize(expr)
+        return TaskRef(finalized._key), (finalized,)
+
+    if type(expr) is type(iter(list())):
+        expr = list(expr)
+    elif type(expr) is type(iter(tuple())):
+        expr = tuple(expr)
+    elif type(expr) is type(iter(set())):
+        expr = set(expr)
+
+    typ = type(expr)
+
+    if typ in (list, tuple, set):
+        args, collections = utils.unzip(
+            (_blockwise_unpack_collections_task_spec(e) for e in expr), 2
+        )
+        collections = tuple(toolz.unique(toolz.concat(collections), key=id))
+        if not collections:
+            return expr, ()
+        args = List(*args)
+        # Ensure output type matches input type
+        if typ is not list:
+            args = Task(None, typ, args)
+        return args, collections
+
+    if typ is dict:
+        args, collections = _blockwise_unpack_collections_task_spec(
+            [[k, v] for k, v in expr.items()]
+        )
+        if not collections:
+            return expr, ()
+        return Dict(args), collections
+
+    if typ is slice:
+        args, collections = _blockwise_unpack_collections_task_spec(
+            [expr.start, expr.stop, expr.step]
+        )
+        if not collections:
+            return expr, ()
+        return Task(None, slice, *args), collections
+
+    if is_dataclass(expr):
+        args, collections = _blockwise_unpack_collections_task_spec(
+            [
+                [f.name, getattr(expr, f.name)]
+                for f in fields(expr)
+                if hasattr(expr, f.name)  # if init=False, field might not exist
+            ]
+        )
+        if not collections:
+            return expr, ()
+        try:
+            _fields = {
+                f.name: getattr(expr, f.name)
+                for f in fields(expr)
+                if hasattr(expr, f.name)
+            }
+            replace(expr, **_fields)
+        except (TypeError, ValueError) as e:
+            if isinstance(e, ValueError) or "is declared with init=False" in str(e):
+                raise ValueError(
+                    f"Failed to unpack {typ} instance. "
+                    "Note that using fields with `init=False` are not supported."
+                ) from e
+            else:
+                raise TypeError(
+                    f"Failed to unpack {typ} instance. "
+                    "Note that using a custom __init__ is not supported."
+                ) from e
+        return Task(None, typ, **dict(args)), collections
+
+    if utils.is_namedtuple_instance(expr):
+        args, collections = _blockwise_unpack_collections_task_spec([v for v in expr])
+        if not collections:
+            return expr, ()
+        return Task(None, typ, *args), collections
+
+    return expr, ()
