@@ -5,12 +5,13 @@ import base64
 import builtins  # Explicitly use builtins.set as 'set' will be shadowed by a function
 import json
 import os
+import pathlib
 import site
 import sys
 import threading
 import warnings
-from collections.abc import Mapping, Sequence
-from typing import Any, Literal
+from collections.abc import Iterator, Mapping, Sequence
+from typing import Any, Literal, overload
 
 import yaml
 
@@ -194,7 +195,21 @@ def _load_config_file(path: str) -> dict | None:
     return config
 
 
-def collect_yaml(paths: Sequence[str] = paths) -> list[dict]:
+@overload
+def collect_yaml(
+    paths: Sequence[str], *, return_paths: Literal[False] = False
+) -> Iterator[dict]: ...
+
+
+@overload
+def collect_yaml(
+    paths: Sequence[str], *, return_paths: Literal[True]
+) -> Iterator[tuple[pathlib.Path, dict]]: ...
+
+
+def collect_yaml(
+    paths: Sequence[str], *, return_paths: bool = False
+) -> Iterator[dict | tuple[pathlib.Path, dict]]:
     """Collect configuration from yaml files
 
     This searches through a list of paths, expands to find all yaml or json
@@ -220,15 +235,14 @@ def collect_yaml(paths: Sequence[str] = paths) -> list[dict]:
             else:
                 file_paths.append(path)
 
-    configs = []
-
     # Parse yaml files
     for path in file_paths:
         config = _load_config_file(path)
         if config is not None:
-            configs.append(config)
-
-    return configs
+            if return_paths:
+                yield pathlib.Path(path), config
+            else:
+                yield config
 
 
 def collect_env(env: Mapping[str, str] | None = None) -> dict:
@@ -257,19 +271,42 @@ def collect_env(env: Mapping[str, str] | None = None) -> dict:
     for name, value in env.items():
         if name.startswith("DASK_"):
             varname = name[5:].lower().replace("__", ".")
-            try:
-                d[varname] = ast.literal_eval(value)
-            except (SyntaxError, ValueError):
-                if value.lower() in ("none", "null"):
-                    d[varname] = None
-                else:
-                    d[varname] = value
+            d[varname] = interpret_value(value)
 
     result: dict = {}
     set(d, config=result)
     return result
 
 
+def interpret_value(value: str) -> Any:
+    try:
+        return ast.literal_eval(value)
+    except (SyntaxError, ValueError):
+        pass
+
+    # Avoid confusion of YAML vs. Python syntax
+    hardcoded_map = {"none": None, "null": None, "false": False, "true": True}
+    return hardcoded_map.get(value.lower(), value)
+
+
+def paths_containing_key(
+    key: str,
+    paths: Sequence[str] = paths,
+) -> Iterator[pathlib.Path]:
+    """
+    Generator yielding paths which contain the given key.
+    """
+    # Check existing config files for any which contains this key.
+    for path_ in paths:
+        for path, config in collect_yaml([path_], return_paths=True):
+            try:
+                get(key, config=config)
+            except KeyError:
+                continue
+            else:
+                yield pathlib.Path(path)
+
+
 def ensure_file(
     source: str, destination: str | None = None, comment: bool = True
 ) -> None:
@@ -376,10 +413,13 @@ class set:
     def __init__(
         self,
         arg: Mapping | None = None,
-        config: dict = config,
+        config: dict | None = None,
         lock: threading.Lock = config_lock,
         **kwargs,
     ):
+        if config is None:  # Keep Sphinx autofunction documentation clean
+            config = global_config
+
         with lock:
             self.config = config
             self._record = []
@@ -480,14 +520,12 @@ def collect(paths: list[str] = paths, env: Mapping[str, str] | None = None) -> d
     if env is None:
         env = os.environ
 
-    configs = collect_yaml(paths=paths)
-    configs.append(collect_env(env=env))
-
+    configs = [*collect_yaml(paths=paths), collect_env(env=env)]
     return merge(*configs)
 
 
 def refresh(
-    config: dict = config, defaults: list[Mapping] = defaults, **kwargs
+    config: dict | None = None, defaults: list[Mapping] = defaults, **kwargs
 ) -> None:
     """
     Update configuration by re-reading yaml files and env variables
@@ -513,6 +551,9 @@ def refresh(
     dask.config.collect: for parameters
     dask.config.update_defaults
     """
+    if config is None:  # Keep Sphinx autofunction documentation clean
+        config = global_config
+
     config.clear()
 
     for d in defaults:
@@ -525,7 +566,7 @@ def refresh(
 def get(
     key: str,
     default: Any = no_default,
-    config: dict = config,
+    config: dict | None = None,
     override_with: Any = None,
 ) -> Any:
     """
@@ -560,6 +601,10 @@ def get(
     """
     if override_with is not None:
         return override_with
+
+    if config is None:  # Keep Sphinx autofunction documentation clean
+        config = global_config
+
     keys = key.split(".")
     result = config
     for k in keys:
@@ -674,6 +719,7 @@ deprecations: dict[str, str | None] = {
     "dataframe.shuffle.algorithm": "dataframe.shuffle.method",
     "dataframe.shuffle-compression": "dataframe.shuffle.compression",
     "admin.traceback.shorten.what": "admin.traceback.shorten",  # changed in 2023.9.0
+    "array.shuffle.chunksize-tolerance": "array.chunk-size-tolerance",
 }
 
 
