@@ -10,6 +10,7 @@ import pandas as pd
 from fsspec.utils import build_name_function, stringify_path
 from tlz import merge
 
+import dask.dataframe as dd
 from dask import config
 from dask.base import (
     compute_as_if_collection,
@@ -227,17 +228,22 @@ def to_hdf(
         lock = get_scheduler_lock(df, scheduler=scheduler)
     elif lock:
         assert isinstance(lock, SupportsLock)
-
-    kwargs.update({"format": "table", "mode": mode, "append": append})
-
     dsk = dict()
 
     i_name = name_function(0)
+    kwargs.update(
+        {
+            "format": "table",
+            "mode": mode,
+            "append": append,
+            "key": key.replace("*", i_name),
+        }
+    )
     dsk[(name, 0)] = (
         _pd_to_hdf,
         pd_to_hdf,
         lock,
-        [(df._name, 0), fmt_obj(path, i_name), key.replace("*", i_name)],
+        [(df._name, 0), fmt_obj(path, i_name)],
         kwargs,
     )
 
@@ -254,12 +260,13 @@ def to_hdf(
 
     for i in range(1, df.npartitions):
         i_name = name_function(i)
+        kwargs2["key"] = key.replace("*", i_name)
         task = (
             _pd_to_hdf,
             pd_to_hdf,
             lock,
-            [(df._name, i), fmt_obj(path, i_name), key.replace("*", i_name)],
-            kwargs2,
+            [(df._name, i), fmt_obj(path, i_name)],
+            kwargs2.copy(),
         )
         if single_file:
             link_dep = i - 1 if single_node else 0
@@ -460,6 +467,17 @@ def read_hdf(
         paths, key, start, stop, chunksize, sorted_index, mode
     )
 
+    if dd._dask_expr_enabled():
+        return dd.from_map(
+            _read_hdf,
+            parts,
+            meta=meta,
+            divisions=divisions,
+            lock=lock,
+            common_kwargs=common_kwargs,
+            columns=columns,
+        )
+
     # Construct the output collection with from_map
     return from_map(
         HDFFunctionWrapper(columns, meta.ndim, lock, common_kwargs),
@@ -472,6 +490,22 @@ def read_hdf(
     )
 
 
+def _read_hdf(part, *, common_kwargs, lock, columns=None):
+    """Read from hdf5 file with a lock"""
+
+    path, key, kwargs = part
+    if lock:
+        lock.acquire()
+    try:
+        result = pd.read_hdf(path, key, **merge(common_kwargs, kwargs))
+    finally:
+        if lock:
+            lock.release()
+    if columns is not None:
+        result = result[columns]
+    return result
+
+
 def _build_parts(paths, key, start, stop, chunksize, sorted_index, mode):
     """
     Build the list of partition inputs and divisions for read_hdf
