@@ -5,21 +5,23 @@ from functools import partial
 from math import ceil
 from operator import getitem
 from threading import Lock
-from typing import TYPE_CHECKING, Iterable, Literal, overload
+from typing import TYPE_CHECKING, Literal, overload
 
 import numpy as np
 import pandas as pd
 
 import dask.array as da
+from dask._task_spec import TaskRef
 from dask.base import is_dask_collection, tokenize
 from dask.blockwise import BlockwiseDepDict, blockwise
+from dask.dataframe._compat import is_any_real_numeric_dtype
 from dask.dataframe.backends import dataframe_creation_dispatch
 from dask.dataframe.core import (
     DataFrame,
-    Index,
     Series,
     _concat,
     _emulate,
+    _Frame,
     apply_and_enforce,
     has_parallel_type,
     new_dd_object,
@@ -54,6 +56,8 @@ def _meta_from_array(x, columns=None, index=None, meta=None):
         )
 
     if index is not None:
+        from dask.dataframe import Index
+
         if not isinstance(index, Index):
             raise ValueError("'index' must be an instance of dask.dataframe.Index")
         index = index._meta
@@ -160,8 +164,7 @@ def from_pandas(
     chunksize: int | None = None,
     sort: bool = True,
     name: str | None = None,
-) -> DataFrame:
-    ...
+) -> DataFrame: ...
 
 
 # We ignore this overload for now until pandas-stubs can be added.
@@ -173,8 +176,7 @@ def from_pandas(  # type: ignore
     chunksize: int | None = None,
     sort: bool = True,
     name: str | None = None,
-) -> Series:
-    ...
+) -> Series: ...
 
 
 def from_pandas(
@@ -278,7 +280,7 @@ def from_pandas(
     if not nrows:
         return new_dd_object({(name, 0): data}, name, data, [None, None])
 
-    if data.index.isna().any() and not data.index.is_numeric():
+    if data.index.isna().any() and not is_any_real_numeric_dtype(data.index):
         raise NotImplementedError(
             "Index in passed data is non-numeric and contains nulls, which Dask does not entirely support.\n"
             "Consider passing `data.loc[~data.isna()]` instead."
@@ -287,12 +289,16 @@ def from_pandas(
     if sort:
         if not data.index.is_monotonic_increasing:
             data = data.sort_index(ascending=True)
+        else:
+            # sort_index copies as well
+            data = data.copy()
         divisions, locations = sorted_division_locations(
             data.index,
             npartitions=npartitions,
             chunksize=chunksize,
         )
     else:
+        data = data.copy()
         if chunksize is None:
             assert isinstance(npartitions, int)
             chunksize = int(ceil(nrows / npartitions))
@@ -488,6 +494,22 @@ def from_dask_array(x, columns=None, index=None, meta=None):
     )
 
     graph = HighLevelGraph.from_collections(name, blk, dependencies=graph_dependencies)
+
+    import dask.dataframe as dd
+
+    if dd._dask_expr_enabled():
+        from dask_expr._collection import from_graph
+
+        from dask.utils import key_split
+
+        return from_graph(
+            graph,
+            meta,
+            divisions,
+            [(name, i) for i in range(len(divisions) - 1)],
+            key_split(name),
+        )
+
     return new_dd_object(graph, name, meta, divisions)
 
 
@@ -584,7 +606,22 @@ def from_delayed(
     prefix: str = "from-delayed",
     verify_meta: bool = True,
 ) -> DataFrame | Series:
-    """Create Dask DataFrame from many Dask Delayed objects
+    """Create Dask DataFrame from many Dask Delayed objects.
+
+    .. warning::
+        ``from_delayed`` should only be used if the objects that create
+        the data are complex and cannot be easily represented as a single
+        function in an embarassingly parallel fashion.
+
+        ``from_map`` is recommended if the query can be expressed as a single
+        function like:
+
+        def read_xml(path):
+            return pd.read_xml(path)
+
+        ddf = dd.from_map(read_xml, paths)
+
+        ``from_delayed`` might be depreacted in the future.
 
     Parameters
     ----------
@@ -639,12 +676,14 @@ def from_delayed(
         name=name,
         columns=None,
         inputs=BlockwiseDepDict(
-            {(i,): inp.key for i, inp in enumerate(dfs)},
+            {(i,): TaskRef(inp.key) for i, inp in enumerate(dfs)},
             produces_keys=True,
         ),
-        io_func=partial(check_meta, meta=meta, funcname="from_delayed")
-        if verify_meta
-        else lambda x: x,
+        io_func=(
+            partial(check_meta, meta=meta, funcname="from_delayed")
+            if verify_meta
+            else lambda x: x
+        ),
     )
     df = new_dd_object(
         HighLevelGraph.from_collections(name, layer, dfs), name, meta, divs
@@ -728,7 +767,7 @@ def sorted_division_locations(seq, npartitions=None, chunksize=None):
         # Map current position selection (i)
         # to the corresponding division value (div)
         div = seq[i]
-        # pos is the position of the first occurance of
+        # pos is the position of the first occurrence of
         # div (which is i when seq has no duplicates)
         if duplicates:
             # Note: cupy requires casts to `int` below
@@ -852,10 +891,13 @@ def from_map(
     enforce_metadata=True,
     **kwargs,
 ):
-    """Create a DataFrame collection from a custom function map
+    """Create a DataFrame collection from a custom function map.
 
-    WARNING: The ``from_map`` API is experimental, and stability is not
-    yet guaranteed. Use at your own risk!
+    ``from_map`` is the preferred option when reading from data sources
+    that are not natively supported by Dask or if the data source
+    requires custom handling before handing things of to Dask DataFrames.
+    Examples are things like binary files or other unstructured data that
+    doesn't have an IO connector.
 
     Parameters
     ----------
@@ -907,13 +949,13 @@ def from_map(
     dtype: object
 
     This API can also be used as an alternative to other file-based
-    IO functions, like ``read_parquet`` (which are already just
+    IO functions, like ``read_csv`` (which are already just
     ``from_map`` wrapper functions):
 
     >>> import pandas as pd
     >>> import dask.dataframe as dd
-    >>> paths = ["0.parquet", "1.parquet", "2.parquet"]
-    >>> dd.from_map(pd.read_parquet, paths).head()  # doctest: +SKIP
+    >>> paths = ["0.csv", "1.csv", "2.csv"]
+    >>> dd.from_map(pd.read_csv, paths).head()  # doctest: +SKIP
                         name
     timestamp
     2000-01-01 00:00:00   Laura
@@ -933,11 +975,11 @@ def from_map(
     >>> import pandas as pd
     >>> import numpy as np
     >>> import dask.dataframe as dd
-    >>> paths = ["0.parquet", "1.parquet", "2.parquet"]
+    >>> paths = ["0.csv", "1.csv", "2.csv"]
     >>> file_sizes = [86400, 86400, 86400]
     >>> def func(path, row_offset):
     ...     # Read parquet file and set RangeIndex offset
-    ...     df = pd.read_parquet(path)
+    ...     df = pd.read_csv(path)
     ...     return df.set_index(
     ...         pd.RangeIndex(row_offset, row_offset+len(df))
     ...     )
@@ -959,7 +1001,6 @@ def from_map(
 
     See Also
     --------
-    dask.dataframe.from_delayed
     dask.layers.DataFrameIOLayer
     """
 
@@ -1075,5 +1116,30 @@ def from_map(
     return new_dd_object(graph, name, meta, divisions)
 
 
+def to_backend(ddf: _Frame, backend: str | None = None, **kwargs):
+    """Move a DataFrame collection to a new backend
+
+    Parameters
+    ----------
+    ddf : DataFrame, Series, or Index
+        The input dataframe collection.
+    backend : str, Optional
+        The name of the new backend to move to. The default
+        is the current "dataframe.backend" configuration.
+
+    Returns
+    -------
+    dask.DataFrame, dask.Series or dask.Index
+        A new dataframe collection with the backend
+        specified by ``backend``.
+    """
+    # Get desired backend
+    backend = backend or dataframe_creation_dispatch.backend
+    # Check that "backend" has a registered entrypoint
+    backend_entrypoint = dataframe_creation_dispatch.dispatch(backend)
+    # Call `DataFrameBackendEntrypoint.to_backend`
+    return backend_entrypoint.to_backend(ddf, **kwargs)
+
+
 DataFrame.to_records.__doc__ = to_records.__doc__
 DataFrame.to_bag.__doc__ = to_bag.__doc__
