@@ -1,13 +1,16 @@
+from __future__ import annotations
+
 import re
 import warnings
 
+import numpy as np
 import pandas as pd
 from fsspec.core import expand_paths_if_needed, get_fs_token_paths, stringify_path
 from fsspec.spec import AbstractFileSystem
 
 from dask import config
 from dask.dataframe.io.utils import _is_local_fs
-from dask.utils import natural_sort_key
+from dask.utils import natural_sort_key, parse_bytes
 
 
 class Engine:
@@ -109,7 +112,8 @@ class Engine:
         paths,
         categories=None,
         index=None,
-        use_nullable_dtypes=False,
+        use_nullable_dtypes=None,
+        dtype_backend=None,
         gather_statistics=None,
         filters=None,
         **kwargs,
@@ -168,6 +172,10 @@ class Engine:
         """
         raise NotImplementedError()
 
+    @classmethod
+    def default_blocksize(cls):
+        return "256 MiB"
+
     @classmethod
     def read_partition(
         cls, fs, piece, columns, index, use_nullable_dtypes=False, **kwargs
@@ -189,6 +197,11 @@ class Engine:
         use_nullable_dtypes: boolean
             Whether to use pandas nullable dtypes (like "string" or "Int64")
             where appropriate when reading parquet files.
+        dtype_backend: {"numpy_nullable", "pyarrow"}
+            Whether to use pandas nullable dtypes (like "string" or "Int64")
+            where appropriate when reading parquet files.
+        convert_string: boolean
+            Whether to use pyarrow strings when reading parquet files.
         **kwargs:
             Includes `"kwargs"` values stored within the `parts` output
             of `engine.read_metadata`. May also include arguments to be
@@ -632,23 +645,38 @@ def _aggregate_stats(
         if len(file_row_group_column_stats) > 1:
             df_cols = pd.DataFrame(file_row_group_column_stats)
         for ind, name in enumerate(stat_col_indices):
-            i = ind * 2
+            i = ind * 3
             if df_cols is None:
-                s["columns"].append(
-                    {
-                        "name": name,
-                        "min": file_row_group_column_stats[0][i],
-                        "max": file_row_group_column_stats[0][i + 1],
-                    }
-                )
+                minval = file_row_group_column_stats[0][i]
+                maxval = file_row_group_column_stats[0][i + 1]
+                null_count = file_row_group_column_stats[0][i + 2]
+                if minval == maxval and null_count:
+                    # Remove "dangerous" stats (min == max, but null values exist)
+                    s["columns"].append({"null_count": null_count})
+                else:
+                    s["columns"].append(
+                        {
+                            "name": name,
+                            "min": minval,
+                            "max": maxval,
+                            "null_count": null_count,
+                        }
+                    )
             else:
-                s["columns"].append(
-                    {
-                        "name": name,
-                        "min": df_cols.iloc[:, i].min(),
-                        "max": df_cols.iloc[:, i + 1].max(),
-                    }
-                )
+                minval = df_cols.iloc[:, i].dropna().min()
+                maxval = df_cols.iloc[:, i + 1].dropna().max()
+                null_count = df_cols.iloc[:, i + 2].sum()
+                if minval == maxval and null_count:
+                    s["columns"].append({"null_count": null_count})
+                else:
+                    s["columns"].append(
+                        {
+                            "name": name,
+                            "min": minval,
+                            "max": maxval,
+                            "null_count": null_count,
+                        }
+                    )
         return s
 
 
@@ -663,7 +691,6 @@ def _row_groups_to_parts(
     make_part_func,
     make_part_kwargs,
 ):
-
     # Construct `parts` and `stats`
     parts = []
     stats = []
@@ -680,7 +707,6 @@ def _row_groups_to_parts(
                 _rgs = list(range(residual, row_group_count, split_row_groups))
 
             for i in _rgs:
-
                 i_end = i + split_row_groups
                 if aggregation_depth is True:
                     if residual and i == 0:
@@ -711,7 +737,6 @@ def _row_groups_to_parts(
                     stats.append(stat)
     else:
         for filename, row_groups in file_row_groups.items():
-
             part = make_part_func(
                 filename,
                 row_groups,
@@ -847,7 +872,7 @@ def _split_user_options(**kwargs):
 
 def _set_gather_statistics(
     gather_statistics,
-    chunksize,
+    blocksize,
     split_row_groups,
     aggregation_depth,
     filter_columns,
@@ -860,7 +885,7 @@ def _set_gather_statistics(
     # If the user has specified `calculate_divisions=True`, then
     # we will be starting with `gather_statistics=True` here.
     if (
-        chunksize
+        (blocksize and split_row_groups is True)
         or (int(split_row_groups) > 1 and aggregation_depth)
         or filter_columns.intersection(stat_columns)
     ):
@@ -876,3 +901,14 @@ def _set_gather_statistics(
         gather_statistics = False
 
     return bool(gather_statistics)
+
+
+def _infer_split_row_groups(row_group_sizes, blocksize, aggregate_files=False):
+    # Use blocksize to choose an appropriate split_row_groups value
+    if row_group_sizes:
+        blocksize = parse_bytes(blocksize)
+        if aggregate_files or np.sum(row_group_sizes) > 2 * blocksize:
+            # If we are aggregating files, or the file is larger
+            # than `blocksize`, set split_row_groups to "adaptive"
+            return "adaptive"
+    return False
