@@ -6,7 +6,7 @@ import pytest
 import dask
 import dask.dataframe as dd
 from dask.blockwise import Blockwise, optimize_blockwise
-from dask.dataframe._compat import PANDAS_GE_200, PANDAS_GE_220, tm
+from dask.dataframe._compat import PANDAS_GE_220, tm
 from dask.dataframe.optimize import optimize_dataframe_getitem
 from dask.dataframe.utils import assert_eq, get_string_dtype
 
@@ -27,7 +27,11 @@ def test_make_timeseries():
     tm.assert_index_equal(df.columns, pd.Index(["A", "B", "C"]))
     assert df["A"].head().dtype == float
     assert df["B"].head().dtype == int
-    assert df["C"].head().dtype == get_string_dtype()
+    assert (
+        df["C"].head().dtype == get_string_dtype()
+        if not dd._dask_expr_enabled()
+        else object
+    )
     assert df.index.name == "timestamp"
     assert df.head().index.name == df.index.name
     assert df.divisions == tuple(pd.date_range(start="2000", end="2015", freq=f"6{ME}"))
@@ -90,6 +94,7 @@ def test_make_timeseries_no_args():
 
 
 @pytest.mark.skip_with_pyarrow_strings  # checks graph layers
+@pytest.mark.skipif(dd._dask_expr_enabled(), reason="layers not supported")
 def test_make_timeseries_blockwise():
     df = dd.demo.make_timeseries()
     df = df[["x", "y"]]
@@ -102,7 +107,7 @@ def test_make_timeseries_blockwise():
     assert set(graph.layers[key].columns) == {"x", "y"}
 
     # Check that `optimize_blockwise` fuses both
-    # `Blockwise` layers together into a singe `Blockwise` layer
+    # `Blockwise` layers together into a single `Blockwise` layer
     graph = optimize_blockwise(df.__dask_graph__(), keys)
     layers = graph.layers
     name = list(layers.keys())[0]
@@ -195,7 +200,9 @@ def test_with_spec(seed):
     assert ddf["i1"].dtype == "int64"
     assert ddf["f1"].dtype == float
     assert ddf["c1"].dtype.name == "category"
-    assert ddf["s1"].dtype == get_string_dtype()
+    assert (
+        ddf["s1"].dtype == get_string_dtype() if not dd._dask_expr_enabled() else object
+    )
     res = ddf.compute()
     assert len(res) == 10
 
@@ -223,12 +230,13 @@ def test_with_spec_non_default(seed):
     ddf = with_spec(spec, seed=seed)
     assert isinstance(ddf, dd.DataFrame)
     assert ddf.columns.tolist() == ["i1", "f1", "c1", "s1"]
-    if PANDAS_GE_200:
-        assert ddf.index.dtype == "int32"
+    assert ddf.index.dtype == "int32"
     assert ddf["i1"].dtype == "int32"
     assert ddf["f1"].dtype == "float32"
     assert ddf["c1"].dtype.name == "category"
-    assert ddf["s1"].dtype == get_string_dtype()
+    assert (
+        ddf["s1"].dtype == get_string_dtype() if not dd._dask_expr_enabled() else object
+    )
     res = ddf.compute().sort_index()
     assert len(res) == 10
     assert set(res.c1.cat.categories) == {"apple", "banana"}
