@@ -2,6 +2,7 @@ from __future__ import annotations
 
 import os
 import pathlib
+from functools import lru_cache
 from time import sleep
 
 import numpy as np
@@ -12,11 +13,12 @@ from packaging.version import Version
 import dask
 import dask.dataframe as dd
 from dask._compatibility import PY_VERSION
+from dask.core import get_deps
 from dask.dataframe._compat import tm
 from dask.dataframe.optimize import optimize_dataframe_getitem
 from dask.dataframe.utils import assert_eq
 from dask.layers import DataFrameIOLayer
-from dask.utils import dependency_depth, tmpdir, tmpfile
+from dask.utils import tmpdir, tmpfile
 
 # there's no support in upstream for writing HDF with extension dtypes yet.
 # see https://github.com/pandas-dev/pandas/issues/31199
@@ -58,6 +60,7 @@ def test_to_hdf():
     PY_VERSION >= Version("3.11"),
     reason="segfaults due to https://github.com/PyTables/PyTables/issues/977",
 )
+@pytest.mark.skipif(dd._dask_expr_enabled(), reason="layers not supported")
 def test_to_hdf_multiple_nodes():
     pytest.importorskip("tables")
     df = pd.DataFrame(
@@ -332,6 +335,20 @@ def test_to_hdf_modes_multiple_files():
         assert_eq(dd.concat([df, df]), out)
 
 
+def dependency_depth(dsk):
+    deps, _ = get_deps(dsk)
+
+    @lru_cache(maxsize=None)
+    def max_depth_by_deps(key):
+        if not deps[key]:
+            return 1
+
+        d = 1 + max(max_depth_by_deps(dep_key) for dep_key in deps[key])
+        return d
+
+    return max(max_depth_by_deps(dep_key) for dep_key in deps.keys())
+
+
 def test_to_hdf_link_optimizations():
     """testing dask link levels is correct by calculating the depth of the dask graph"""
     pytest.importorskip("tables")
@@ -654,7 +671,7 @@ def test_to_fmt_warns():
 def test_read_hdf(data, compare):
     pytest.importorskip("tables")
     with tmpfile("h5") as fn:
-        data.to_hdf(fn, "/data")
+        data.to_hdf(fn, key="/data")
         try:
             dd.read_hdf(fn, "data", chunksize=2, mode="r")
             assert False
@@ -662,7 +679,7 @@ def test_read_hdf(data, compare):
             assert "format='table'" in str(e)
 
     with tmpfile("h5") as fn:
-        data.to_hdf(fn, "/data", format="table")
+        data.to_hdf(fn, key="/data", format="table")
         a = dd.read_hdf(fn, "/data", chunksize=2, mode="r")
         assert a.npartitions == 2
 
@@ -679,7 +696,7 @@ def test_read_hdf(data, compare):
 
     with tmpfile("h5") as fn:
         sorted_data = data.sort_index()
-        sorted_data.to_hdf(fn, "/data", format="table")
+        sorted_data.to_hdf(fn, key="/data", format="table")
         a = dd.read_hdf(fn, "/data", chunksize=2, sorted_index=True, mode="r")
         assert a.npartitions == 2
 
@@ -694,7 +711,7 @@ def test_read_hdf_multiply_open():
         {"x": ["a", "b", "c", "d"], "y": [1, 2, 3, 4]}, index=[1.0, 2.0, 3.0, 4.0]
     )
     with tmpfile("h5") as fn:
-        df.to_hdf(fn, "/data", format="table")
+        df.to_hdf(fn, key="/data", format="table")
         with pd.HDFStore(fn, mode="r"):
             dd.read_hdf(fn, "/data", chunksize=2, mode="r")
 
@@ -762,7 +779,7 @@ def test_read_hdf_start_stop_values():
         {"x": ["a", "b", "c", "d"], "y": [1, 2, 3, 4]}, index=[1.0, 2.0, 3.0, 4.0]
     )
     with tmpfile("h5") as fn:
-        df.to_hdf(fn, "/data", format="table")
+        df.to_hdf(fn, key="/data", format="table")
 
         with pytest.raises(ValueError, match="number of rows"):
             dd.read_hdf(fn, "/data", stop=10)
@@ -781,9 +798,9 @@ def test_hdf_globbing():
     )
 
     with tmpdir() as tdir:
-        df.to_hdf(os.path.join(tdir, "one.h5"), "/foo/data", format="table")
-        df.to_hdf(os.path.join(tdir, "two.h5"), "/bar/data", format="table")
-        df.to_hdf(os.path.join(tdir, "two.h5"), "/foo/data", format="table")
+        df.to_hdf(os.path.join(tdir, "one.h5"), key="/foo/data", format="table")
+        df.to_hdf(os.path.join(tdir, "two.h5"), key="/bar/data", format="table")
+        df.to_hdf(os.path.join(tdir, "two.h5"), key="/foo/data", format="table")
 
         with dask.config.set(scheduler="sync"):
             res = dd.read_hdf(os.path.join(tdir, "one.h5"), "/*/data", chunksize=2)
@@ -818,8 +835,12 @@ def test_hdf_file_list():
     )
 
     with tmpdir() as tdir:
-        df.iloc[:2].to_hdf(os.path.join(tdir, "one.h5"), "dataframe", format="table")
-        df.iloc[2:].to_hdf(os.path.join(tdir, "two.h5"), "dataframe", format="table")
+        df.iloc[:2].to_hdf(
+            os.path.join(tdir, "one.h5"), key="dataframe", format="table"
+        )
+        df.iloc[2:].to_hdf(
+            os.path.join(tdir, "two.h5"), key="dataframe", format="table"
+        )
 
         with dask.config.set(scheduler="sync"):
             input_files = [os.path.join(tdir, "one.h5"), os.path.join(tdir, "two.h5")]
@@ -835,7 +856,7 @@ def test_read_hdf_pattern_pathlike():
 
     with tmpfile("h5") as fn:
         path = pathlib.Path(fn)
-        df.to_hdf(path, "dataframe", format="table")
+        df.to_hdf(path, key="dataframe", format="table")
         res = dd.read_hdf(path, "dataframe")
         assert_eq(res, df)
 
@@ -925,7 +946,7 @@ def test_hdf_nonpandas_keys():
 
         # pandas keys should still work
         bar = pd.DataFrame(np.random.randn(10, 4))
-        bar.to_hdf(path, "/bar", format="table", mode="a")
+        bar.to_hdf(path, key="/bar", format="table", mode="a")
 
         dd.read_hdf(path, "/group/table1")
         dd.read_hdf(path, "/group/table2")
