@@ -11,13 +11,15 @@ import dask.array as da
 import dask.dataframe as dd
 from dask import config
 from dask.blockwise import Blockwise
-from dask.dataframe._compat import PANDAS_GE_200, tm
+from dask.dataframe._compat import tm
 from dask.dataframe.io.io import _meta_from_array
 from dask.dataframe.optimize import optimize
 from dask.dataframe.utils import assert_eq, get_string_dtype, pyarrow_strings_enabled
 from dask.delayed import Delayed, delayed
 from dask.utils_test import hlg_layer_topological
 
+DASK_EXPR_ENABLED = dd._dask_expr_enabled()
+
 ##########
 # Arrays #
 ##########
@@ -124,7 +126,7 @@ def test_from_pandas_dataframe():
         index=pd.date_range(start="20120101", periods=len(a)),
     )
     ddf = dd.from_pandas(df, 3)
-    expected_layers = 6 if pyarrow_strings_enabled() else 3
+    expected_layers = 6 if pyarrow_strings_enabled() and not DASK_EXPR_ENABLED else 3
     assert len(ddf.dask) == expected_layers
     assert len(ddf.divisions) == 4
     assert isinstance(ddf.divisions[0], type(df.index[0]))
@@ -135,9 +137,10 @@ def test_from_pandas_dataframe():
     with pytest.raises(ValueError) as err:
         dd.from_pandas(df, npartitions=2, chunksize=2)
     assert msg in str(err.value)
-    with pytest.raises((ValueError, AssertionError)) as err:
-        dd.from_pandas(df)
-    assert msg in str(err.value)
+    if not DASK_EXPR_ENABLED:
+        with pytest.raises((ValueError, AssertionError)) as err:
+            dd.from_pandas(df)
+        assert msg in str(err.value)
     assert len(ddf.dask) == expected_layers
     assert len(ddf.divisions) == 4
     assert isinstance(ddf.divisions[0], type(df.index[0]))
@@ -245,10 +248,11 @@ def test_from_pandas_with_wrong_args():
     df = pd.DataFrame({"x": [1, 2, 3]}, index=[3, 2, 1])
     with pytest.raises(TypeError, match="must be a pandas DataFrame or Series"):
         dd.from_pandas("foo")
-    with pytest.raises(
-        ValueError, match="one of npartitions and chunksize must be specified"
-    ):
-        dd.from_pandas(df)
+    if not DASK_EXPR_ENABLED:
+        with pytest.raises(
+            ValueError, match="one of npartitions and chunksize must be specified"
+        ):
+            dd.from_pandas(df)
     with pytest.raises(TypeError, match="provide npartitions as an int"):
         dd.from_pandas(df, npartitions=5.2, sort=False)
     with pytest.raises(TypeError, match="provide chunksize as an int"):
@@ -279,12 +283,8 @@ def test_from_pandas_npartitions_duplicates(index):
     assert ddf.divisions == ("A", "B", "C", "C")
 
 
-@pytest.mark.skipif(
-    not PANDAS_GE_200, reason="dataframe.convert-string requires pandas>=2.0"
-)
 def test_from_pandas_convert_string_config():
     pytest.importorskip("pyarrow", reason="Requires pyarrow strings")
-
     # With `dataframe.convert-string=False`, strings should remain objects
     with dask.config.set({"dataframe.convert-string": False}):
         s = pd.Series(["foo", "bar", "ricky", "bobby"], index=["a", "b", "c", "d"])
@@ -317,24 +317,6 @@ def test_from_pandas_convert_string_config():
     assert_eq(df_pyarrow, ddf)
 
 
-@pytest.mark.skipif(PANDAS_GE_200, reason="Requires pandas<2.0")
-def test_from_pandas_convert_string_config_raises():
-    pytest.importorskip("pyarrow", reason="Different error without pyarrow")
-    df = pd.DataFrame(
-        {
-            "x": [1, 2, 3, 4],
-            "y": [5.0, 6.0, 7.0, 8.0],
-            "z": ["foo", "bar", "ricky", "bobby"],
-        },
-        index=["a", "b", "c", "d"],
-    )
-    with dask.config.set({"dataframe.convert-string": True}):
-        with pytest.raises(
-            RuntimeError, match="requires `pandas>=2.0` to be installed"
-        ):
-            dd.from_pandas(df, npartitions=2)
-
-
 @pytest.mark.parametrize("index", [[1, 2, 3], [3, 2, 1]])
 @pytest.mark.parametrize("sort", [True, False])
 def test_from_pandas_immutable(sort, index):
@@ -359,12 +341,14 @@ def test_DataFrame_from_dask_array():
     x = da.ones((10, 3), chunks=(4, 2))
     pdf = pd.DataFrame(np.ones((10, 3)), columns=["a", "b", "c"])
     df = dd.from_dask_array(x, ["a", "b", "c"])
-    assert not hlg_layer_topological(df.dask, -1).is_materialized()
+    if not DASK_EXPR_ENABLED:
+        assert not hlg_layer_topological(df.dask, -1).is_materialized()
     assert_eq(df, pdf)
 
     # dd.from_array should re-route to from_dask_array
     df2 = dd.from_array(x, columns=["a", "b", "c"])
-    assert not hlg_layer_topological(df2.dask, -1).is_materialized()
+    if not DASK_EXPR_ENABLED:
+        assert not hlg_layer_topological(df2.dask, -1).is_materialized()
     assert_eq(df, df2)
 
 
@@ -375,10 +359,11 @@ def test_DataFrame_from_dask_array_with_blockwise_ops():
     df = dd.from_dask_array(x, ["a", "b", "c"])
     # None of the layers in this graph should be materialized, everything should
     # be a HighLevelGraph still.
-    assert all(
-        not hlg_layer_topological(df.dask, i).is_materialized()
-        for i in range(len(df.dask.layers))
-    )
+    if not DASK_EXPR_ENABLED:
+        assert all(
+            not hlg_layer_topological(df.dask, i).is_materialized()
+            for i in range(len(df.dask.layers))
+        )
     assert_eq(df, pdf)
 
 
@@ -707,7 +692,8 @@ def test_from_delayed():
     meta2 = [(c, "f8") for c in df.columns]
     # Make sure `from_delayed` is Blockwise
     check_ddf = dd.from_delayed(dfs, meta=meta2)
-    assert isinstance(check_ddf.dask.layers[check_ddf._name], Blockwise)
+    if not DASK_EXPR_ENABLED:
+        assert isinstance(check_ddf.dask.layers[check_ddf._name], Blockwise)
     assert_eq(dd.from_delayed(dfs, meta=meta2), df)
     assert_eq(dd.from_delayed([d.a for d in dfs], meta=("a", "f8")), df.a)
 
@@ -719,6 +705,7 @@ def test_from_delayed():
     assert str(e.value).startswith("Metadata mismatch found in `from_delayed`")
 
 
+@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="doesn't make sense")
 def test_from_delayed_optimize_fusion():
     # Test that DataFrame optimization fuses a `from_delayed`
     # layer with other Blockwise layers and input Delayed tasks.
@@ -759,6 +746,7 @@ def test_from_delayed_to_dask_array():
     assert result.shape == (9, 2)
 
 
+@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="doesn't make sense")
 def test_from_delayed_preserves_hlgs():
     df = pd.DataFrame(data=np.random.normal(size=(10, 4)), columns=list("abcd"))
     parts = [df.iloc[:1], df.iloc[1:3], df.iloc[3:6], df.iloc[6:10]]
@@ -797,6 +785,7 @@ def test_from_delayed_misordered_meta():
     assert msg in str(info.value)
 
 
+@pytest.mark.xfail(DASK_EXPR_ENABLED, reason="not supported")
 def test_from_delayed_sorted():
     a = pd.DataFrame({"x": [1, 2]}, index=[1, 10])
     b = pd.DataFrame({"x": [4, 1]}, index=[100, 200])
@@ -833,7 +822,8 @@ def test_to_delayed_optimize_graph():
     d = ddf2.to_delayed()[0]
     assert len(d.dask) < 20
     d2 = ddf2.to_delayed(optimize_graph=False)[0]
-    assert sorted(d2.dask) == sorted(ddf2.dask)
+    if not dd._dask_expr_enabled():
+        assert sorted(d2.dask) == sorted(ddf2.dask)
     assert_eq(ddf2.get_partition(0), d.compute())
     assert_eq(ddf2.get_partition(0), d2.compute())
 
@@ -841,7 +831,8 @@ def test_to_delayed_optimize_graph():
     x = ddf2.x.sum()
     dx = x.to_delayed()
     dx2 = x.to_delayed(optimize_graph=False)
-    assert len(dx.dask) < len(dx2.dask)
+    if not dd._dask_expr_enabled():
+        assert len(dx.dask) < len(dx2.dask)
     assert_eq(dx.compute(), dx2.compute())
 
 
@@ -895,13 +886,16 @@ def test_from_map_simple(vals):
         index=[1, 1, 2, 2],
     )
 
-    # Make sure `from_map` produces single `Blockwise` layer
-    layers = ser.dask.layers
-    expected_layers = (
-        2 if pyarrow_strings_enabled() and any(isinstance(v, str) for v in vals) else 1
-    )
-    assert len(layers) == expected_layers
-    assert isinstance(layers[ser._name], Blockwise)
+    if not DASK_EXPR_ENABLED:
+        # Make sure `from_map` produces single `Blockwise` layer
+        layers = ser.dask.layers
+        expected_layers = (
+            2
+            if pyarrow_strings_enabled() and any(isinstance(v, str) for v in vals)
+            else 1
+        )
+        assert len(layers) == expected_layers
+        assert isinstance(layers[ser._name], Blockwise)
 
     # Check that result and partition count make sense
     assert ser.npartitions == len(iterable)
@@ -974,8 +968,10 @@ def test_from_map_meta():
     meta = pd.DataFrame({"a": pd.Series(["A"], dtype=string_dtype)}).iloc[:0]
     ddf = dd.from_map(func, iterable, meta=meta, s=2)
     assert_eq(ddf._meta, meta)
-    with pytest.raises(ValueError, match="The columns in the computed data"):
-        assert_eq(ddf.compute(), expect)
+    if not DASK_EXPR_ENABLED:
+        # no validation yet
+        with pytest.raises(ValueError, match="The columns in the computed data"):
+            assert_eq(ddf.compute(), expect)
 
     # Third Check - Pass in invalid metadata,
     # but use `enforce_metadata=False`
@@ -984,6 +980,7 @@ def test_from_map_meta():
     assert_eq(ddf.compute(), expect)
 
 
+@pytest.mark.skipif(DASK_EXPR_ENABLED, reason="dask-expr doesn't support token")
 def test_from_map_custom_name():
     # Test that `label` and `token` arguments to
     # `from_map` works as expected
@@ -1032,29 +1029,31 @@ def test_from_map_other_iterables(iterable):
     assert_eq(ddf.compute(), expect)
 
 
+class MyFunc:
+    projected: list[str] = []
+
+    def __init__(self, columns=None):
+        self.columns = columns
+
+    def project_columns(self, columns):
+        return MyFunc(columns)
+
+    def __call__(self, t, columns=None):
+        cols = self.columns or columns
+        size = t[0] + 1
+        x = t[1]
+        df = pd.DataFrame({"A": [x] * size, "B": [10] * size})
+        if cols is None:
+            return df
+        MyFunc.projected.extend(cols)
+        return df[cols]
+
+
 def test_from_map_column_projection():
     # Test that column projection works
     # as expected with from_map when
     # enforce_metadata=True
 
-    projected = []
-
-    class MyFunc:
-        def __init__(self, columns=None):
-            self.columns = columns
-
-        def project_columns(self, columns):
-            return MyFunc(columns)
-
-        def __call__(self, t):
-            size = t[0] + 1
-            x = t[1]
-            df = pd.DataFrame({"A": [x] * size, "B": [10] * size})
-            if self.columns is None:
-                return df
-            projected.extend(self.columns)
-            return df[self.columns]
-
     ddf = dd.from_map(
         MyFunc(),
         enumerate([0, 1, 2]),
@@ -1069,7 +1068,7 @@ def test_from_map_column_projection():
         index=[0, 0, 1, 0, 1, 2],
     )
     assert_eq(ddf["A"], expect["A"])
-    assert set(projected) == {"A"}
+    assert set(MyFunc.projected) == {"A"}
     assert_eq(ddf, expect)
 
 
