@@ -1,6 +1,7 @@
 from __future__ import annotations
 
 import datetime
+import warnings
 from numbers import Integral
 
 import pandas as pd
@@ -9,7 +10,7 @@ from pandas.core.window import Rolling as pd_Rolling
 
 from dask.array.core import normalize_arg
 from dask.base import tokenize
-from dask.blockwise import BlockwiseDepDict
+from dask.blockwise import BlockwiseDepDict, _blockwise_unpack_collections_task_spec
 from dask.dataframe import methods
 from dask.dataframe._compat import check_axis_keyword_deprecation
 from dask.dataframe.core import (
@@ -30,7 +31,6 @@ from dask.dataframe.utils import (
     is_dataframe_like,
     is_series_like,
 )
-from dask.delayed import unpack_collections
 from dask.highlevelgraph import HighLevelGraph
 from dask.typing import no_default
 from dask.utils import M, apply, derived_from, funcname, has_keyword
@@ -111,7 +111,7 @@ def map_overlap(
     ----------
     func : function
         The function applied to each partition. If this function accepts
-        the special ``partition_info`` keyword argument, it will recieve
+        the special ``partition_info`` keyword argument, it will receive
         information on the partition's relative location within the
         dataframe.
     df: dd.DataFrame, dd.Series
@@ -254,7 +254,7 @@ def map_overlap(
             dependencies.append(arg)
             continue
         arg = normalize_arg(arg)
-        arg2, collections = unpack_collections(arg)
+        arg2, collections = _blockwise_unpack_collections_task_spec(arg)
         if collections:
             args2.append(arg2)
             dependencies.extend(collections)
@@ -265,7 +265,7 @@ def map_overlap(
     simple = True
     for k, v in kwargs.items():
         v = normalize_arg(v)
-        v, collections = unpack_collections(v)
+        v, collections = _blockwise_unpack_collections_task_spec(v)
         dependencies.extend(collections)
         kwargs3[k] = v
         if collections:
@@ -479,6 +479,13 @@ class Rolling:
         # See https://github.com/pandas-dev/pandas/issues/15969
         self._win_type = None if isinstance(self.window, int) else "freq"
 
+        if self.axis in ("index", 1, "rows"):
+            warnings.warn(
+                "Using axis=1 in Rolling has been deprecated and will be removed "
+                "in a future version.",
+                FutureWarning,
+            )
+
     def _rolling_kwargs(self):
         kwargs = {
             "window": self.window,
