@@ -4,6 +4,8 @@ import warnings
 
 import pytest
 
+from dask.array.numpy_compat import NUMPY_GE_200
+
 pd = pytest.importorskip("pandas")
 
 import numpy as np
@@ -12,10 +14,6 @@ import dask.array as da
 import dask.dataframe as dd
 from dask.dataframe.utils import assert_eq
 
-if dd._dask_expr_enabled():
-    pytest.skip("doesn't work", allow_module_level=True)
-
-
 _BASE_UFUNCS = [
     "conj",
     "exp",
@@ -457,6 +455,10 @@ def test_mixed_types(ufunc, arg1, arg2):
     assert_eq(dafunc(arg2, arg1), npfunc(arg2, arg1))
 
 
+@pytest.mark.skipif(
+    dd._dask_expr_enabled(),
+    reason="doesn't work at the moment, all return not implemented",
+)
 @pytest.mark.parametrize("ufunc", _UFUNCS_2ARG)
 @pytest.mark.parametrize(
     "pandas,darray",
@@ -519,6 +521,14 @@ def test_ufunc_with_reduction(redfunc, ufunc, pandas):
     np_redfunc = getattr(np, redfunc)
     np_ufunc = getattr(np, ufunc)
 
+    if (
+        NUMPY_GE_200
+        and redfunc == "prod"
+        and ufunc in ("floor", "ceil", "trunc")
+        and isinstance(pandas, pd.DataFrame)
+    ):
+        pytest.skip("Numpy started overflowing while we are casting to float")
+
     if (
         redfunc == "prod"
         and ufunc in ["conj", "square", "negative", "absolute"]
@@ -533,7 +543,14 @@ def test_ufunc_with_reduction(redfunc, ufunc, pandas):
     with warnings.catch_warnings():
         warnings.simplefilter("ignore", RuntimeWarning)
         warnings.simplefilter("ignore", FutureWarning)
-        assert isinstance(np_redfunc(dask), (dd.DataFrame, dd.Series, dd.core.Scalar))
+        if dd._dask_expr_enabled():
+            import dask_expr as dx
+
+            assert isinstance(np_redfunc(dask), (dd.DataFrame, dd.Series, dx.Scalar))
+        else:
+            assert isinstance(
+                np_redfunc(dask), (dd.DataFrame, dd.Series, dd.core.Scalar)
+            )
         assert_eq(np_redfunc(np_ufunc(dask)), np_redfunc(np_ufunc(pandas)))
 
 
