@@ -11,7 +11,7 @@ def timeseries(
     start="2000-01-01",
     end="2000-01-31",
     freq="1s",
-    partition_freq="1d",
+    partition_freq="1D",
     dtypes=None,
     seed=None,
     **kwargs,
@@ -50,7 +50,7 @@ def timeseries(
     2000-01-01 00:00:04   998   Ursula  0.684902 -0.463278
     >>> df = dask.datasets.timeseries(
     ...     '2000', '2010',
-    ...     freq='2H', partition_freq='1D', seed=1,  # data frequency
+    ...     freq='2h', partition_freq='1D', seed=1,  # data frequency
     ...     dtypes={'value': float, 'name': str, 'id': int},  # data types
     ...     id_lam=1000  # control number of items in id column
     ... )
@@ -95,7 +95,7 @@ def _generate_mimesis(field, schema_description, records_per_partition, seed):
 
 def _make_mimesis(field, schema, npartitions, records_per_partition, seed=None):
     """
-    Make a Dask Bag filled with data randomly generated by the mimesis projet
+    Make a Dask Bag filled with data randomly generated by the mimesis project
 
     Parameters
     ----------
@@ -163,7 +163,7 @@ def make_people(npartitions=10, records_per_partition=1000, seed=None, locale="e
     )
 
     schema = lambda field: {
-        "age": field("person.age"),
+        "age": field("random.randint", a=0, b=120),
         "name": (field("person.name"), field("person.surname")),
         "occupation": field("person.occupation"),
         "telephone": field("person.telephone"),
