@@ -6,23 +6,21 @@ import operator
 from collections import defaultdict
 from collections.abc import Callable
 from itertools import product
-from typing import Any
+from typing import TYPE_CHECKING, Any
 
 import tlz as toolz
 from tlz.curried import map
 
+from dask._task_spec import Task, TaskRef
 from dask.base import tokenize
 from dask.blockwise import Blockwise, BlockwiseDep, BlockwiseDepDict, blockwise_token
-from dask.core import flatten, keys_in_tasks
+from dask.core import flatten
 from dask.highlevelgraph import Layer
-from dask.utils import (
-    apply,
-    cached_cumsum,
-    concrete,
-    insert,
-    stringify,
-    stringify_collection_keys,
-)
+from dask.tokenize import normalize_token
+from dask.utils import apply, cached_cumsum, concrete, insert
+
+if TYPE_CHECKING:
+    import numpy as np
 
 #
 ##
@@ -72,15 +70,6 @@ class ArrayBlockwiseDep(BlockwiseDep):
     def __getitem__(self, idx: tuple[int, ...]):
         raise NotImplementedError("Subclasses must implement __getitem__")
 
-    def __dask_distributed_pack__(
-        self, required_indices: list[tuple[int, ...]] | None = None
-    ):
-        return {"chunks": self.chunks}
-
-    @classmethod
-    def __dask_distributed_unpack__(cls, state):
-        return cls(**state)
-
 
 class ArrayChunkShapeDep(ArrayBlockwiseDep):
     """Produce chunk shapes given a chunk index"""
@@ -103,6 +92,38 @@ class ArraySliceDep(ArrayBlockwiseDep):
         return tuple(slice(*s, None) for s in loc)
 
 
+class ArrayBlockIdDep(ArrayBlockwiseDep):
+    def __getitem__(self, idx: tuple):
+        if not isinstance(idx, tuple):
+            # This happens if something else except graph generation calls us
+            raise NotImplementedError("ArrayBlockIdDep requires a tuple index")
+        return idx
+
+
+class ArrayValuesDep(ArrayBlockwiseDep):
+    def __init__(self, chunks: tuple[tuple[int, ...], ...], values: np.ndarray | dict):
+        super().__init__(chunks)
+        self.values = values
+
+    def __getitem__(self, idx: tuple):
+        return self.values[idx]
+
+
+@normalize_token.register(ArraySliceDep)
+def normalize_array_slice_dep(dep):
+    return "ArraySliceDep", dep.chunks
+
+
+@normalize_token.register(ArrayBlockIdDep)
+def normalize_array_block_id_dep(dep):
+    return "ArrayBlockIdDep", dep.chunks
+
+
+@normalize_token.register(ArrayValuesDep)
+def normalize_array_values_dep(dep):
+    return "ArrayValuesDep", dep.chunks, dep.values
+
+
 class ArrayOverlapLayer(Layer):
     """Simple HighLevelGraph array overlap layer.
 
@@ -212,6 +233,8 @@ class ArrayOverlapLayer(Layer):
         overlap_blocks = {}
         for k in interior_keys:
             frac_slice = fractional_slice((name,) + k, axes)
+            if frac_slice is False:
+                continue
             if (name,) + k != frac_slice:
                 interior_slices[(getitem_name,) + k] = frac_slice
             else:
@@ -224,18 +247,14 @@ class ArrayOverlapLayer(Layer):
         dsk = toolz.merge(interior_slices, overlap_blocks)
         return dsk
 
-    @classmethod
-    def __dask_distributed_unpack__(cls, state):
-        return cls(**state)._construct_graph(deserializing=True)
-
 
 def _expand_keys_around_center(k, dims, name=None, axes=None):
     """Get all neighboring keys around center
 
     Parameters
     ----------
-    k: tuple
-        They key around which to generate new keys
+    k: Key
+        The key around which to generate new keys
     dims: Sequence[int]
         The number of chunks in each dimension
     name: Option[str]
@@ -255,31 +274,45 @@ def _expand_keys_around_center(k, dims, name=None, axes=None):
      [('y', 0.9, 3.1), ('y', 0.9,   4)]]
     """
 
-    def inds(i, ind):
+    def convert_depth(depth):
+        if not isinstance(depth, tuple):
+            depth = (depth, depth)
+        return depth
+
+    def inds(i, ind, depth):
+        depth = convert_depth(depth)
         rv = []
-        if ind - 0.9 > 0:
+        if ind - 0.9 > 0 and depth[0] != 0:
             rv.append(ind - 0.9)
         rv.append(ind)
-        if ind + 0.9 < dims[i] - 1:
+        if ind + 0.9 < dims[i] - 1 and depth[1] != 0:
             rv.append(ind + 0.9)
         return rv
 
     shape = []
     for i, ind in enumerate(k[1:]):
+        depth = convert_depth(axes.get(i, 0))
         num = 1
-        if ind > 0:
+        if ind > 0 and depth[0] != 0:
             num += 1
-        if ind < dims[i] - 1:
+        if ind < dims[i] - 1 and depth[1] != 0:
             num += 1
         shape.append(num)
 
+    def _valid_depth(depth):
+        if isinstance(depth, tuple):
+            return any(x != 0 for x in depth)
+        else:
+            return depth != 0
+
     args = [
-        inds(i, ind) if any((axes.get(i, 0),)) else [ind] for i, ind in enumerate(k[1:])
+        inds(i, ind, axes.get(i, 0)) if _valid_depth(axes.get(i, 0)) else [ind]
+        for i, ind in enumerate(k[1:])
     ]
     if name is not None:
         args = [[name]] + args
     seq = list(product(*args))
-    shape2 = [d if any((axes.get(i, 0),)) else 1 for i, d in enumerate(shape)]
+    shape2 = [d if _valid_depth(axes.get(i, 0)) else 1 for i, d in enumerate(shape)]
     result = reshapelist(shape2, seq)
     return result
 
@@ -328,7 +361,7 @@ def fractional_slice(task, axes):
         elif t > r and left_depth:
             index.append(slice(-left_depth, None))
         else:
-            index.append(slice(0, 0))
+            return False
     index = tuple(index)
 
     if all(ind == slice(None, None, None) for ind in index):
@@ -385,7 +418,6 @@ class SimpleShuffleLayer(Layer):
         parts_out=None,
         annotations=None,
     ):
-        super().__init__(annotations=annotations)
         self.name = name
         self.column = column
         self.npartitions = npartitions
@@ -409,19 +441,19 @@ class SimpleShuffleLayer(Layer):
         # much earlier.
         #
         # See https://github.com/dask/dask/pull/6051 for a detailed discussion.
-        self.annotations = self.annotations or {}
-        if "priority" not in self.annotations:
-            self.annotations["priority"] = {}
-        self.annotations["priority"]["__expanded_annotations__"] = None
-        self.annotations["priority"].update({_key: 1 for _key in self.get_split_keys()})
+        annotations = annotations or {}
+        self._split_keys = None
+        if "priority" not in annotations:
+            annotations["priority"] = self._key_priority
 
-    def get_split_keys(self):
-        # Return SimpleShuffleLayer "split" keys
-        return [
-            stringify((self.split_name, part_out, part_in))
-            for part_in in range(self.npartitions_input)
-            for part_out in self.parts_out
-        ]
+        super().__init__(annotations=annotations)
+
+    def _key_priority(self, key):
+        assert isinstance(key, tuple)
+        if key[0] == self.split_name:
+            return 1
+        else:
+            return 0
 
     def get_output_keys(self):
         return {(self.name, part) for part in self.parts_out}
@@ -509,61 +541,6 @@ class SimpleShuffleLayer(Layer):
         else:
             return self, culled_deps
 
-    def __reduce__(self):
-        attrs = [
-            "name",
-            "column",
-            "npartitions",
-            "npartitions_input",
-            "ignore_index",
-            "name_input",
-            "meta_input",
-            "parts_out",
-            "annotations",
-        ]
-        return (SimpleShuffleLayer, tuple(getattr(self, attr) for attr in attrs))
-
-    def __dask_distributed_pack__(
-        self, all_hlg_keys, known_key_dependencies, client, client_keys
-    ):
-        from distributed.protocol.serialize import to_serialize
-
-        return {
-            "name": self.name,
-            "column": self.column,
-            "npartitions": self.npartitions,
-            "npartitions_input": self.npartitions_input,
-            "ignore_index": self.ignore_index,
-            "name_input": self.name_input,
-            "meta_input": to_serialize(self.meta_input),
-            "parts_out": list(self.parts_out),
-        }
-
-    @classmethod
-    def __dask_distributed_unpack__(cls, state, dsk, dependencies):
-        from distributed.worker import dumps_task
-
-        # msgpack will convert lists into tuples, here
-        # we convert them back to lists
-        if isinstance(state["column"], tuple):
-            state["column"] = list(state["column"])
-        if "inputs" in state:
-            state["inputs"] = list(state["inputs"])
-
-        # Materialize the layer
-        layer_dsk = cls(**state)._construct_graph(deserializing=True)
-
-        # Convert all keys to strings and dump tasks
-        layer_dsk = {
-            stringify(k): stringify_collection_keys(v) for k, v in layer_dsk.items()
-        }
-        keys = layer_dsk.keys() | dsk.keys()
-
-        # TODO: use shuffle-knowledge to calculate dependencies more efficiently
-        deps = {k: keys_in_tasks(keys, [v]) for k, v in layer_dsk.items()}
-
-        return {"dsk": toolz.valmap(dumps_task, layer_dsk), "deps": deps}
-
     def _construct_graph(self, deserializing=False):
         """Construct graph for a simple shuffle operation."""
 
@@ -680,53 +657,11 @@ class ShuffleLayer(SimpleShuffleLayer):
             annotations=annotations,
         )
 
-    def get_split_keys(self):
-        # Return ShuffleLayer "split" keys
-        keys = []
-        for part in self.parts_out:
-            out = self.inputs[part]
-            for i in range(self.nsplits):
-                keys.append(
-                    stringify(
-                        (
-                            self.split_name,
-                            out[self.stage],
-                            insert(out, self.stage, i),
-                        )
-                    )
-                )
-        return keys
-
     def __repr__(self):
         return "ShuffleLayer<name='{}', stage={}, nsplits={}, npartitions={}>".format(
             self.name, self.stage, self.nsplits, self.npartitions
         )
 
-    def __reduce__(self):
-        attrs = [
-            "name",
-            "column",
-            "inputs",
-            "stage",
-            "npartitions",
-            "npartitions_input",
-            "nsplits",
-            "ignore_index",
-            "name_input",
-            "meta_input",
-            "parts_out",
-            "annotations",
-        ]
-
-        return (ShuffleLayer, tuple(getattr(self, attr) for attr in attrs))
-
-    def __dask_distributed_pack__(self, *args, **kwargs):
-        ret = super().__dask_distributed_pack__(*args, **kwargs)
-        ret["inputs"] = self.inputs
-        ret["stage"] = self.stage
-        ret["nsplits"] = self.nsplits
-        return ret
-
     def _cull_dependencies(self, keys, parts_out=None):
         """Determine the necessary dependencies to produce `keys`.
 
@@ -781,7 +716,6 @@ class ShuffleLayer(SimpleShuffleLayer):
         dsk = {}
         inp_part_map = {inp: i for i, inp in enumerate(self.inputs)}
         for part in self.parts_out:
-
             out = self.inputs[part]
 
             _concat_list = []  # get_item tasks to concat for this output partition
@@ -806,7 +740,6 @@ class ShuffleLayer(SimpleShuffleLayer):
                 )
 
                 if (shuffle_group_name, _inp) not in dsk:
-
                     # Initial partitions (output of previous stage)
                     _part = inp_part_map[_inp]
                     if self.stage == 0:
@@ -872,6 +805,8 @@ class BroadcastJoinLayer(Layer):
         rhs_npartitions,
         parts_out=None,
         annotations=None,
+        left_on=None,
+        right_on=None,
         **merge_kwargs,
     ):
         super().__init__(annotations=annotations)
@@ -882,14 +817,12 @@ class BroadcastJoinLayer(Layer):
         self.rhs_name = rhs_name
         self.rhs_npartitions = rhs_npartitions
         self.parts_out = parts_out or set(range(self.npartitions))
+        self.left_on = tuple(left_on) if isinstance(left_on, list) else left_on
+        self.right_on = tuple(right_on) if isinstance(right_on, list) else right_on
         self.merge_kwargs = merge_kwargs
         self.how = self.merge_kwargs.get("how")
-        self.left_on = self.merge_kwargs.get("left_on")
-        self.right_on = self.merge_kwargs.get("right_on")
-        if isinstance(self.left_on, list):
-            self.left_on = (list, tuple(self.left_on))
-        if isinstance(self.right_on, list):
-            self.right_on = (list, tuple(self.right_on))
+        self.merge_kwargs["left_on"] = self.left_on
+        self.merge_kwargs["right_on"] = self.right_on
 
     def get_output_keys(self):
         return {(self.name, part) for part in self.parts_out}
@@ -921,47 +854,6 @@ class BroadcastJoinLayer(Layer):
     def __len__(self):
         return len(self._dict)
 
-    def __dask_distributed_pack__(self, *args, **kwargs):
-        import pickle
-
-        # Pickle complex merge_kwargs elements. Also
-        # tuples, which may be confused with keys.
-        _merge_kwargs = {}
-        for k, v in self.merge_kwargs.items():
-            if not isinstance(v, (str, list, bool)):
-                _merge_kwargs[k] = pickle.dumps(v)
-            else:
-                _merge_kwargs[k] = v
-
-        return {
-            "name": self.name,
-            "npartitions": self.npartitions,
-            "lhs_name": self.lhs_name,
-            "lhs_npartitions": self.lhs_npartitions,
-            "rhs_name": self.rhs_name,
-            "rhs_npartitions": self.rhs_npartitions,
-            "parts_out": self.parts_out,
-            "merge_kwargs": _merge_kwargs,
-        }
-
-    @classmethod
-    def __dask_distributed_unpack__(cls, state, dsk, dependencies):
-        from distributed.worker import dumps_task
-
-        # Expand merge_kwargs
-        merge_kwargs = state.pop("merge_kwargs", {})
-        state.update(merge_kwargs)
-
-        # Materialize the layer
-        raw = cls(**state)._construct_graph(deserializing=True)
-
-        # Convert all keys to strings and dump tasks
-        raw = {stringify(k): stringify_collection_keys(v) for k, v in raw.items()}
-        keys = raw.keys() | dsk.keys()
-        deps = {k: keys_in_tasks(keys, [v]) for k, v in raw.items()}
-
-        return {"dsk": toolz.valmap(dumps_task, raw), "deps": deps}
-
     def _keys_to_parts(self, keys):
         """Simple utility to convert keys to partition indices."""
         parts = set()
@@ -1081,7 +973,6 @@ class BroadcastJoinLayer(Layer):
         # any output partitions that are not requested (via `parts_out`)
         dsk = {}
         for i in self.parts_out:
-
             # Split each "other" partition by hash
             if self.how != "inner":
                 dsk[(split_name, i)] = (
@@ -1103,12 +994,14 @@ class BroadcastJoinLayer(Layer):
                 # Specify arg list for `merge_chunk`
                 _merge_args = [
                     (
-                        operator.getitem,
-                        (split_name, i),
-                        j,
-                    )
-                    if self.how != "inner"
-                    else (other_name, i),
+                        (
+                            operator.getitem,
+                            (split_name, i),
+                            j,
+                        )
+                        if self.how != "inner"
+                        else (other_name, i)
+                    ),
                     (bcast_name, j),
                 ]
                 if bcast_side == "left":
@@ -1197,11 +1090,11 @@ class DataFrameIOLayer(Blockwise):
             io_arg_map = inputs
 
         # Use Blockwise initializer
-        dsk = {self.name: (io_func, blockwise_token(0))}
+        task = Task(self.name, io_func, TaskRef(blockwise_token(0)))
         super().__init__(
             output=self.name,
             output_indices="i",
-            dsk=dsk,
+            task=task,
             indices=[(io_arg_map, "i")],
             numblocks={},
             annotations=annotations,
@@ -1222,7 +1115,6 @@ class DataFrameIOLayer(Blockwise):
         columns = list(columns)
 
         if self.columns is None or set(self.columns).issuperset(columns):
-
             # Apply column projection in IO function.
             # Must satisfy `DataFrameIOFunction` protocol
             if isinstance(self.io_func, DataFrameIOFunction):
@@ -1512,49 +1404,3 @@ class DataFrameTreeReduction(Layer):
             return culled_layer, deps
         else:
             return self, deps
-
-    def __dask_distributed_pack__(self, *args, **kwargs):
-        from distributed.protocol.serialize import to_serialize
-
-        # Pickle the (possibly) user-defined functions here
-        _concat_func = to_serialize(self.concat_func)
-        _tree_node_func = to_serialize(self.tree_node_func)
-        if self.finalize_func:
-            _finalize_func = to_serialize(self.finalize_func)
-        else:
-            _finalize_func = None
-
-        return {
-            "name": self.name,
-            "name_input": self.name_input,
-            "npartitions_input": self.npartitions_input,
-            "concat_func": _concat_func,
-            "tree_node_func": _tree_node_func,
-            "finalize_func": _finalize_func,
-            "split_every": self.split_every,
-            "split_out": self.split_out,
-            "output_partitions": self.output_partitions,
-            "tree_node_name": self.tree_node_name,
-        }
-
-    @classmethod
-    def __dask_distributed_unpack__(cls, state, dsk, dependencies):
-        from distributed.protocol.serialize import to_serialize
-
-        # Materialize the layer
-        raw = cls(**state)._construct_graph()
-
-        # Convert all keys to strings and dump tasks
-        raw = {stringify(k): stringify_collection_keys(v) for k, v in raw.items()}
-        keys = raw.keys() | dsk.keys()
-        deps = {k: keys_in_tasks(keys, [v]) for k, v in raw.items()}
-
-        # Must use `to_serialize` on the entire task.
-        # This is required because the task-tuples contain `Serialized`
-        # function objects instead of real functions. Using `dumps_task`
-        # may or may not correctly wrap the entire tuple in `to_serialize`.
-        # So we use `to_serialize` here to be explicit. When the task
-        # arrives at a worker, both the `Serialized` task-tuples and the
-        # `Serialized` functions nested within them should be deserialzed
-        # automatically by the comm.
-        return {"dsk": toolz.valmap(to_serialize, raw), "deps": deps}
