@@ -106,6 +106,7 @@ significantly on space and computation complexity.
 
 See the function ``inline_functions`` for more information.
 """
+
 from __future__ import annotations
 
 import os
@@ -115,8 +116,9 @@ from functools import partial
 from queue import Empty, Queue
 
 from dask import config
+from dask._task_spec import DataNode, DependenciesMapping, convert_legacy_graph
 from dask.callbacks import local_callbacks, unpack_callbacks
-from dask.core import _execute_task, flatten, get_dependencies, has_tasks, reverse_dict
+from dask.core import flatten, get_dependencies, reverse_dict
 from dask.order import order
 from dask.typing import Key
 
@@ -164,17 +166,19 @@ def start_state_from_dask(dsk, cache=None, sortkey=None):
         cache = config.get("cache", None)
     if cache is None:
         cache = dict()
+
+    dsk = convert_legacy_graph(dsk, all_keys=set(dsk) | set(cache))
     data_keys = set()
     for k, v in dsk.items():
-        if not has_tasks(dsk, v):
-            cache[k] = v
+        if isinstance(v, DataNode):
+            cache[k] = v()
             data_keys.add(k)
 
     dsk2 = dsk.copy()
     dsk2.update(cache)
 
-    dependencies = {k: get_dependencies(dsk2, k) for k in dsk}
-    waiting = {k: v.copy() for k, v in dependencies.items() if k not in data_keys}
+    dependencies = DependenciesMapping(dsk)
+    waiting = {k: set(v) for k, v in dependencies.items() if k not in data_keys}
 
     dependents = reverse_dict(dependencies)
     for a in cache:
@@ -222,11 +226,11 @@ def execute_task(key, task_info, dumps, loads, get_id, pack_exception):
     """
     try:
         task, data = loads(task_info)
-        result = _execute_task(task, data)
+        result = task(data)
         id = get_id()
         result = dumps((result, id))
         failed = False
-    except BaseException as e:
+    except BaseException as e:  # noqa: B036
         result = pack_exception(e, dumps)
         failed = True
     return key, result, failed
@@ -422,7 +426,7 @@ def get_async(
         result_flat = {result}
     results = set(result_flat)
 
-    dsk = dict(dsk)
+    dsk = dict(convert_legacy_graph(dsk))
     with local_callbacks(callbacks) as callbacks:
         _, _, pretask_cbs, posttask_cbs, _ = unpack_callbacks(callbacks)
         started_cbs = []
@@ -507,7 +511,7 @@ def get_async(
                                 for dep in get_dependencies(dsk, key)
                             }
                             task = dsk[key]
-                            _execute_task(task, data)  # Re-execute locally
+                            task(data)  # Re-execute locally
                         else:
                             raise_exception(exc, tb)
                     res, worker_id = loads(res_info)
@@ -541,7 +545,7 @@ class SynchronousExecutor(Executor):
         fut = Future()
         try:
             fut.set_result(fn(*args, **kwargs))
-        except BaseException as e:
+        except BaseException as e:  # noqa: B036
             fut.set_exception(e)
         return fut
 
