@@ -47,10 +47,11 @@ arise, and the order we would like to be determined.
 
 """
 from collections import defaultdict, deque, namedtuple
-from collections.abc import Iterable, Mapping, MutableMapping
-from typing import Any, Callable, Literal, NamedTuple, overload
+from collections.abc import Callable, Iterable, Mapping, MutableMapping
+from typing import Any, Literal, NamedTuple, overload
 
-from dask.core import get_dependencies, get_deps, getcycle, istask, reverse_dict
+from dask._task_spec import DataNode, DependenciesMapping
+from dask.core import get_deps, getcycle, istask, reverse_dict
 from dask.typing import Key
 
 
@@ -65,8 +66,7 @@ def order(
     dependencies: Mapping[Key, set[Key]] | None = None,
     *,
     return_stats: Literal[True],
-) -> dict[Key, Order]:
-    ...
+) -> dict[Key, Order]: ...
 
 
 @overload
@@ -75,8 +75,7 @@ def order(
     dependencies: Mapping[Key, set[Key]] | None = None,
     *,
     return_stats: Literal[False] = False,
-) -> dict[Key, int]:
-    ...
+) -> dict[Key, int]: ...
 
 
 def order(
@@ -103,23 +102,24 @@ def order(
     {'a': 0, 'c': 1, 'b': 2, 'd': 3}
     """
     if not dsk:
-        return {}  # type: ignore
+        return {}
 
     dsk = dict(dsk)
-    expected_len = len(dsk)
-
-    if dependencies is None:
-        dependencies = {k: get_dependencies(dsk, k) for k in dsk}
 
+    dependencies = DependenciesMapping(dsk)
     dependents = reverse_dict(dependencies)
-    num_needed, total_dependencies = ndependencies(dependencies, dependents)
-    if len(total_dependencies) != len(dsk):
-        cycle = getcycle(dsk, None)
-        raise RuntimeError(
-            "Cycle detected between the following keys:\n  -> %s"
-            % "\n  -> ".join(str(x) for x in cycle)
-        )
 
+    external_keys = set()
+    if len(dependents) != len(dependencies):
+        # There are external keys. Let's add a DataNode to ensure the algo below
+        # is encountering a complete graph. Those artificial data nodes will be
+        # ignored when assigning results
+        for k in dependents:
+            if k not in dependencies:
+                external_keys.add(k)
+                dsk[k] = DataNode(k, object())
+
+    expected_len = len(dsk)
     leaf_nodes = {k for k, v in dependents.items() if not v}
     root_nodes = {k for k, v in dependencies.items() if not v}
 
@@ -128,11 +128,21 @@ def order(
     # Normalize the graph by removing leaf nodes that are not actual tasks, see
     # for instance da.store where the task is merely an alias
     # to multiple keys, i.e. [key1, key2, ...,]
+
+    # Similarly, we are removing root nodes that are pure data tasks. Those task
+    # are embedded in the run_spec of a task and are not runnable. We have to
+    # assign a priority but their priority has no impact on performance.
+    # The removal of those tasks typically transforms the graph topology in a
+    # way that is simpler to handle
     all_tasks = False
     n_removed_leaves = 0
+    requires_data_task = defaultdict(set)
+
     while not all_tasks:
         all_tasks = True
         for leaf in list(leaf_nodes):
+            if leaf in root_nodes:
+                continue
             if (
                 not istask(dsk[leaf])
                 # Having a linear chain is fine
@@ -148,13 +158,35 @@ def order(
                     result[leaf] = prio
                 n_removed_leaves += 1
                 leaf_nodes.remove(leaf)
-                del dsk[leaf]
-                del dependents[leaf]
                 for dep in dependencies[leaf]:
                     dependents[dep].remove(leaf)
                     if not dependents[dep]:
                         leaf_nodes.add(dep)
+                del dsk[leaf]
+                del dependencies[leaf]
+                del dependents[leaf]
 
+        for root in list(root_nodes):
+            if root in leaf_nodes:
+                continue
+            deps_root = dependents[root]
+            if not istask(dsk[root]) and len(deps_root) > 1:
+                del dsk[root]
+                del dependencies[root]
+                root_nodes.remove(root)
+                del dependents[root]
+                for dep in deps_root:
+                    requires_data_task[dep].add(root)
+                    if not dependencies[dep]:
+                        root_nodes.add(dep)
+
+    num_needed, total_dependencies = ndependencies(dependencies, dependents)
+    if len(total_dependencies) != len(dsk):
+        cycle = getcycle(dsk, None)
+        raise RuntimeError(
+            "Cycle detected between the following keys:\n  -> %s"
+            % "\n  -> ".join(str(x) for x in cycle)
+        )
     assert dependencies is not None
     roots_connected, max_dependents = _connecting_to_roots(dependencies, dependents)
     leafs_connected, _ = _connecting_to_roots(dependents, dependencies)
@@ -174,14 +206,17 @@ def order(
 
     _sort_keys_cache: dict[Key, tuple[int, int, int, int, str]] = {}
 
+    leafs_connected_to_loaded_roots: set[Key] = set()
+    processed_roots = set()
+
     def sort_key(x: Key) -> tuple[int, int, int, int, str]:
         try:
             return _sort_keys_cache[x]
         except KeyError:
             assert dependencies is not None
             _sort_keys_cache[x] = rv = (
-                len(dependencies[x]),
                 total_dependencies[x],
+                len(dependencies[x]),
                 len(roots_connected[x]),
                 -max_dependents[x],
                 # Converting to str is actually faster than creating some
@@ -199,23 +234,28 @@ def order(
         nonlocal i
         while next_items:
             item = next_items.pop()
-            assert not num_needed[item]
             runnable_hull.discard(item)
             reachable_hull.discard(item)
             leaf_nodes.discard(item)
             if item in result:
                 continue
+
+            while requires_data_task[item]:
+                add_to_result(requires_data_task[item].pop())
             if return_stats:
                 result[item] = Order(i, crit_path_counter - _crit_path_counter_offset)
             else:
                 result[item] = i
-            i += 1
+            if item not in external_keys:
+                i += 1
+            if item in root_nodes:
+                processed_roots.add(item)
             # Note: This is a `set` and therefore this introduces a certain
             # randomness. However, this randomness should not have any impact on
             # the final result since the `process_runnable` should produce
             # equivalent results regardless of the order in which runnable is
             # populated (not identical but equivalent)
-            for dep in dependents[item]:
+            for dep in dependents.get(item, ()):
                 num_needed[dep] -= 1
                 reachable_hull.add(dep)
                 if not num_needed[dep]:
@@ -250,7 +290,7 @@ def order(
         such free tasks too early we'd be walking the graph in a too wide /
         breadth first fashion that is not optimal. If instead we were to only
         execute them once they are needed for a final result, this can cause
-        very high memory pressure since valueable reducers are executed too
+        very high memory pressure since valuable reducers are executed too
         late.
 
         The strategy here is to take all runnable tasks and walk forwards until
@@ -260,7 +300,7 @@ def order(
         reducer to be runnable, we will execute the path.
 
         If instead of a reducer a splitter is encountered that is runnable, we
-        will follow it's splitter paths individually and apply the same logic to
+        will follow its splitter paths individually and apply the same logic to
         each branch.
         """
         while runnable:
@@ -274,14 +314,17 @@ def order(
                     add_to_result(key)
                     continue
                 path = [key]
-                branches = deque([path])
+                branches = deque([(0, path)])
+
                 while branches:
-                    path = branches.popleft()
+                    nsplits, path = branches.popleft()
                     while True:
+                        # Loop invariant. Too expensive to compute at runtime
+                        # assert not set(known_runnable_paths).intersection(runnable_hull)
                         current = path[-1]
                         runnable_hull.add(current)
                         deps_downstream = dependents[current]
-                        deps_upstream = dependencies[current]  # type: ignore
+                        deps_upstream = dependencies[current]
                         if not deps_downstream:
                             # FIXME: The fact that it is possible for
                             # num_needed[current] == 0 means we're doing some
@@ -289,22 +332,25 @@ def order(
                             if num_needed[current] <= 1:
                                 for k in path:
                                     add_to_result(k)
+                            else:
+                                runnable_hull.discard(current)
                         elif len(path) == 1 or len(deps_upstream) == 1:
                             if len(deps_downstream) > 1:
+                                nsplits += 1
                                 for d in sorted(deps_downstream, key=sort_key):
                                     # This ensures we're only considering splitters
                                     # that are genuinely splitting and not
                                     # interleaving
-                                    if len(dependencies[d]) == 1:  # type: ignore
+                                    if len(dependencies[d]) == 1:
                                         branch = path.copy()
                                         branch.append(d)
-                                        branches.append(branch)
+                                        branches.append((nsplits, branch))
                                 break
-                            runnable_hull.update(deps_downstream)
-                            path.extend(sorted(deps_downstream, key=sort_key))
+                            path.extend(deps_downstream)
                             continue
                         elif current in known_runnable_paths:
                             known_runnable_paths[current].append(path)
+                            runnable_hull.discard(current)
                             if (
                                 len(known_runnable_paths[current])
                                 >= num_needed[current]
@@ -318,6 +364,13 @@ def order(
                                         pruned_branches
                                     )
                                 else:
+                                    if nsplits > 1:
+                                        path = []
+                                        for pruned in pruned_branches:
+                                            path.extend(pruned)
+                                        branches.append((nsplits - 1, path))
+                                        break
+
                                     while pruned_branches:
                                         path = pruned_branches.popleft()
                                         for k in path:
@@ -327,13 +380,14 @@ def order(
                                             add_to_result(k)
                         else:
                             if (
-                                len(dependencies[current]) > 1  # type: ignore
+                                len(dependencies[current]) > 1
                                 and num_needed[current] <= 1
                             ):
                                 for k in path:
                                     add_to_result(k)
                             else:
                                 known_runnable_paths[current] = [path]
+                                runnable_hull.discard(current)
                         break
 
     # Pick strategy
@@ -344,6 +398,43 @@ def order(
     # writing, the most expensive part of ordering is the prep work (mostly
     # connected roots + sort_key) which can be reused for multiple orderings.
 
+    def get_target(longest_path: bool = False) -> Key:
+        # Some topologies benefit if the node with the most dependencies
+        # is used as first choice, others benefit from the opposite.
+        candidates = leaf_nodes
+        skey: Callable = sort_key
+        if reachable_hull:
+            skey = lambda k: (num_needed[k], sort_key(k))
+
+        all_leafs_accessible = len(leafs_connected_to_loaded_roots) >= len(candidates)
+        if reachable_hull and not all_leafs_accessible:
+            # Avoid this branch if we can already access all leafs. Just pick
+            # one of them without the expensive selection process in this case.
+
+            candidates = reachable_hull & candidates
+            if not candidates:
+                # We can't reach a leaf node directly, but we still have nodes
+                # with results in memory, these notes can inform our path towards
+                # a new preferred leaf node.
+                for r in processed_roots:
+                    leafs_connected_to_loaded_roots.update(leafs_connected[r])
+                processed_roots.clear()
+
+                leafs_connected_to_loaded_roots.intersection_update(leaf_nodes)
+                candidates = leafs_connected_to_loaded_roots
+            else:
+                leafs_connected_to_loaded_roots.update(candidates)
+
+        elif not reachable_hull:
+            # We reach a new and independent branch, so throw away previous branch
+            leafs_connected_to_loaded_roots.clear()
+
+        if longest_path and (not reachable_hull or all_leafs_accessible):
+            return leaf_nodes_sorted.pop()
+        else:
+            # FIXME: This can be very expensive
+            return min(candidates, key=skey)
+
     def use_longest_path() -> bool:
         size = 0
         # Heavy reducer / splitter topologies often benefit from a very
@@ -362,63 +453,9 @@ def order(
         return True
 
     longest_path = use_longest_path()
-
-    def get_target() -> Key:
-        raise NotImplementedError()
-
-    if not longest_path:
-
-        def _build_get_target() -> Callable[[], Key]:
-            occurences: defaultdict[Key, int] = defaultdict(int)
-            for t in leaf_nodes:
-                for r in roots_connected[t]:
-                    occurences[r] += 1
-            occurences_grouped = defaultdict(set)
-            for root, occ in occurences.items():
-                occurences_grouped[occ].add(root)
-            occurences_grouped_sorted = {}
-            for k, v in occurences_grouped.items():
-                occurences_grouped_sorted[k] = sorted(v, key=sort_key, reverse=True)
-            del occurences_grouped, occurences
-
-            def pick_seed() -> Key | None:
-                while occurences_grouped_sorted:
-                    key = max(occurences_grouped_sorted)
-                    picked_root = occurences_grouped_sorted[key][-1]
-                    if picked_root in result:
-                        occurences_grouped_sorted[key].pop()
-                        if not occurences_grouped_sorted[key]:
-                            del occurences_grouped_sorted[key]
-                        continue
-                    return picked_root
-                return None
-
-            def get_target() -> Key:
-                candidates = leaf_nodes
-                skey: Callable = sort_key
-
-                if runnable_hull:
-                    skey = lambda k: (num_needed[k], sort_key(k))
-                    candidates = runnable_hull & candidates
-                elif reachable_hull:
-                    skey = lambda k: (num_needed[k], sort_key(k))
-                    candidates = reachable_hull & candidates
-
-                if not candidates:
-                    if seed := pick_seed():
-                        candidates = leafs_connected[seed]
-                    else:
-                        candidates = runnable_hull or reachable_hull
-                # FIXME: This can be very expensive
-                return min(candidates, key=skey)
-
-            return get_target
-
-        get_target = _build_get_target()
-    else:
+    leaf_nodes_sorted = []
+    if longest_path:
         leaf_nodes_sorted = sorted(leaf_nodes, key=sort_key, reverse=False)
-        get_target = leaf_nodes_sorted.pop
-        del leaf_nodes_sorted
 
     # *************************************************************************
     # CORE ALGORITHM STARTS HERE
@@ -453,9 +490,9 @@ def order(
     #   can define any route through the graph that should be considered as top
     #   priority.
     #
-    #   1. Determine the target node by calling `get_target`` and append the
+    #   1. Determine the target node by calling ``get_target`` and append the
     #      target to the critical path stack
-    #   2. Take the _most valuable_ (max given a `sort_key`) of it's dependends
+    #   2. Take the _most valuable_ (max given a `sort_key`) of its dependents
     #      and append it to the critical path stack. This key is the new target.
     #   3. Repeat step 2 until we reach a node that has no dependencies and is
     #      therefore runnable
@@ -518,14 +555,14 @@ def order(
         assert not scrit_path
 
         # A. Build the critical path
-        target = get_target()
+        target = get_target(longest_path=longest_path)
         next_deps = dependencies[target]
         path_append(target)
 
         while next_deps:
             item = max(next_deps, key=sort_key)
             path_append(item)
-            next_deps = dependencies[item]
+            next_deps = dependencies[item].difference(result)
             path_extend(next_deps)
 
         # B. Walk the critical path
@@ -536,10 +573,6 @@ def order(
             if item in result:
                 continue
             if num_needed[item]:
-                if item in known_runnable_paths:
-                    for path in known_runnable_paths_pop(item):
-                        path_extend(reversed(path))
-                    continue
                 path_append(item)
                 deps = dependencies[item].difference(result)
                 unknown: list[Key] = []
@@ -569,12 +602,14 @@ def order(
         process_runnables()
 
     assert len(result) == expected_len
+    for k in external_keys:
+        del result[k]
     return result  # type: ignore
 
 
 def _connecting_to_roots(
     dependencies: Mapping[Key, set[Key]], dependents: Mapping[Key, set[Key]]
-) -> tuple[dict[Key, set[Key]], dict[Key, int]]:
+) -> tuple[dict[Key, frozenset[Key]], dict[Key, int]]:
     """Determine for every node which root nodes are connected to it (i.e.
     ancestors). If arguments of dependencies and dependents are switched, this
     can also be used to determine which leaf nodes are connected to which node
@@ -599,47 +634,67 @@ def _connecting_to_roots(
             # benefit from the speedup of using integers would be to convert
             # this back on demand which makes the code very hard to read.
             roots.add(k)
-            result[k] = {k}
+            result[k] = frozenset({k})
             deps = dependents[k]
             max_dependents[k] = len(deps)
             for child in deps:
                 num_needed[child] -= 1
                 if not num_needed[child]:
                     current.append(child)
+
+    dedup_mapping: dict[frozenset[Key], frozenset[Key]] = {}
     while current:
         key = current.pop()
+        if key in result:
+            continue
         for parent in dependents[key]:
             num_needed[parent] -= 1
             if not num_needed[parent]:
                 current.append(parent)
         # At some point, all the roots are the same, particularly for dense
         # graphs. We don't want to create new sets over and over again
-        new_set = None
-        identical_sets = True
-        result_first = None
-
+        transitive_deps = []
+        transitive_deps_ids = set()
+        max_dependents_key = list()
         for child in dependencies[key]:
             r_child = result[child]
-            if not result_first:
-                result_first = r_child
-                max_dependents[key] = max_dependents[child]
-            # This clause is written such that it can circuit break early
-            elif not (  # type: ignore[unreachable]
-                identical_sets
-                and (result_first is r_child or r_child.issubset(result_first))
-            ):
-                identical_sets = False
-                if not new_set:
-                    new_set = result_first.copy()
-                max_dependents[key] = max(max_dependents[child], max_dependents[key])
-                new_set.update(r_child)
+            if id(r_child) in transitive_deps_ids:
+                continue
+            transitive_deps.append(r_child)
+            transitive_deps_ids.add(id(r_child))
+            max_dependents_key.append(max_dependents[child])
 
-        assert new_set is not None or result_first is not None
-        result[key] = new_set or result_first
+        max_dependents[key] = max(max_dependents_key)
+        if len(transitive_deps_ids) == 1:
+            result[key] = transitive_deps[0]
+        else:
+            prev: set | frozenset | None = None
+            new_set = None
+            for tdeps in transitive_deps:
+                if prev is None:
+                    prev = tdeps
+                    continue
+                if tdeps.issubset(prev):
+                    continue
+                if new_set is None:
+                    prev = new_set = set(prev)
+                prev.update(tdeps)
+            if new_set is not None:
+                # frozenset is unfortunately triggering a copy. In the event of
+                # a cache hit, this is wasted time but we can't hash the set
+                # otherwise (unless we did it manually) and can therefore not
+                # deduplicate without this copy
+                frozen_res = frozenset(new_set)
+                try:
+                    result[key] = dedup_mapping[frozen_res]
+                except KeyError:
+                    dedup_mapping[frozen_res] = frozen_res
+                    result[key] = frozen_res
+            else:
+                result[key] = prev  # type: ignore
+    del dedup_mapping
 
-    # The order algo doesn't care about this but this makes it easier to
-    # understand and shouldn't take that much time
-    empty_set: set[Key] = set()
+    empty_set: frozenset[Key] = frozenset()
     for r in roots:
         result[r] = empty_set
     return result, max_dependents
@@ -766,40 +821,17 @@ def diagnostics(
     return rv, pressure
 
 
-def _f() -> None:
-    ...
-
-
-def _convert_task(task: Any) -> Any:
-    if istask(task):
-        assert callable(task[0])
-        new_spec: list[Any] = []
-        for el in task[1:]:
-            if isinstance(el, (str, int)):
-                new_spec.append(el)
-            elif isinstance(el, tuple):
-                if istask(el):
-                    new_spec.append(_convert_task(el))
-                else:
-                    new_spec.append(el)
-            elif isinstance(el, list):
-                new_spec.append([_convert_task(e) for e in el])
-        return (_f, *new_spec)
-    elif isinstance(task, tuple):
-        return (_f, task)
-    else:
-        return (_f, *task)
+def _f() -> None: ...
 
 
 def sanitize_dsk(dsk: MutableMapping[Key, Any]) -> dict:
     """Take a dask graph and replace callables with a dummy function and remove
     payload data like numpy arrays, dataframes, etc.
     """
+    from dask._task_spec import Task, TaskRef
+
     new = {}
-    for key, values in dsk.items():
-        new_key = key
-        new[new_key] = _convert_task(values)
-    if get_deps(new) != get_deps(dsk):
-        # The switch statement in _convert likely dropped some keys
-        raise RuntimeError("Sanitization failed to preserve topology.")
+    deps = DependenciesMapping(dsk)
+    for key, values in deps.items():
+        new[key] = Task(key, _f, *(TaskRef(k) for k in values))
     return new
