@@ -6,9 +6,9 @@ import random
 import sys
 from array import array
 
-import importlib_metadata
-from packaging.version import parse as parse_version
+from packaging.version import Version
 
+from dask._compatibility import importlib_metadata
 from dask.utils import Dispatch
 
 sizeof = Dispatch(name="sizeof")
@@ -213,10 +213,7 @@ def register_pandas():
 
     @sizeof.register(pd.MultiIndex)
     def sizeof_pandas_multiindex(i):
-        p = sum(sizeof(lev) for lev in i.levels)
-        for c in i.codes:
-            p += c.nbytes
-        return p
+        return sum(sizeof(l) for l in i.levels) + sum(c.nbytes for c in i.codes)
 
 
 @sizeof.register_lazy("scipy")
@@ -224,7 +221,7 @@ def register_spmatrix():
     import scipy
     from scipy import sparse
 
-    if parse_version(scipy.__version__) < parse_version("1.12.0.dev0"):
+    if Version(scipy.__version__) < Version("1.12.0.dev0"):
 
         @sizeof.register(sparse.dok_matrix)
         def sizeof_spmatrix_dok(s):
@@ -261,6 +258,51 @@ def register_pyarrow():
         return int(_get_col_size(data)) + 1000
 
 
+@sizeof.register_lazy("xarray")
+def register_xarray():
+    import sys
+
+    import xarray as xr
+
+    XARRAY_VERSION = Version(xr.__version__)
+    XARRAY_GE_2024_02 = XARRAY_VERSION >= Version("2024.02.0")
+
+    @sizeof.register(xr.core.utils.Frozen)
+    def xarray_sizeof_frozen(obj):
+        return sys.getsizeof(obj) + sizeof(obj.mapping)
+
+    @sizeof.register(xr.DataArray)
+    @sizeof.register(xr.Variable)
+    def xarray_sizeof_da(obj):
+        return sys.getsizeof(obj) + sizeof(obj.data)
+
+    @sizeof.register(xr.Dataset)
+    def xarray_sizeof_ds(obj):
+        return sys.getsizeof(obj) + sizeof(obj.variables)
+
+    if XARRAY_GE_2024_02:
+        xarray_sizeof_da = sizeof.register(xr.NamedArray)(xarray_sizeof_da)
+
+    @sizeof.register(xr.core.indexes.Indexes)
+    def xarray_sizeof_indexes(obj):
+        return (
+            sys.getsizeof(obj)
+            + sizeof(obj._index_type)
+            + sizeof(obj._indexes)
+            + sizeof(obj._variables)
+            + sizeof(obj._dims)
+        )
+
+    @sizeof.register(xr.core.indexes.PandasIndex)
+    def xarray_sizeof_pd_index(obj):
+        return (
+            sys.getsizeof(obj)
+            + sizeof(obj.index)
+            + sizeof(obj.dim)
+            + sizeof(obj.coord_dtype)
+        )
+
+
 def _register_entry_point_plugins():
     """Register sizeof implementations exposed by the entry_point mechanism."""
     for entry_point in importlib_metadata.entry_points(group="dask.sizeof"):
