@@ -9,15 +9,15 @@ import pytest
 
 from dask.datasets import timeseries
 
-dd = pytest.importorskip("dask.dataframe")
+pytest.importorskip("pandas")
 pyspark = pytest.importorskip("pyspark")
 pa = pytest.importorskip("pyarrow")
 pytest.importorskip("fastparquet")
+dd = pytest.importorskip("dask.dataframe")
 
 import numpy as np
 import pandas as pd
 
-from dask.dataframe._compat import PANDAS_GE_150, PANDAS_GE_200
 from dask.dataframe.utils import assert_eq
 
 pytestmark = [
@@ -25,14 +25,27 @@ pytestmark = [
         sys.platform != "linux",
         reason="Unnecessary, and hard to get spark working on non-linux platforms",
     ),
-    pytest.mark.skipif(
-        PANDAS_GE_200,
+    pytest.mark.skip(
         reason="pyspark doesn't yet have support for pandas 2.0",
     ),
     # we only test with pyarrow strings and pandas 2.0
     pytest.mark.skip_with_pyarrow_strings,  # pyspark doesn't support pandas 2.0
 ]
 
+
+@pytest.fixture(
+    params=[
+        "pyarrow",
+        pytest.param(
+            "fastparquet", marks=pytest.mark.filterwarnings("ignore::FutureWarning")
+        ),
+    ]
+)
+def engine(request):
+    pytest.importorskip(request.param)
+    return request.param
+
+
 # pyspark auto-converts timezones -- round-tripping timestamps is easier if
 # we set everything to UTC.
 pdf = timeseries(freq="1h").compute()
@@ -62,8 +75,7 @@ def spark_session():
         signal.signal(signal.SIGINT, prev)
 
 
-@pytest.mark.parametrize("npartitions", (1, 5, 10))
-@pytest.mark.parametrize("engine", ("pyarrow", "fastparquet"))
+@pytest.mark.parametrize("npartitions", [1, 5, 10])
 def test_roundtrip_parquet_spark_to_dask(spark_session, npartitions, tmpdir, engine):
     tmpdir = str(tmpdir)
 
@@ -80,7 +92,6 @@ def test_roundtrip_parquet_spark_to_dask(spark_session, npartitions, tmpdir, eng
     assert_eq(ddf, pdf, check_index=False)
 
 
-@pytest.mark.parametrize("engine", ("pyarrow", "fastparquet"))
 def test_roundtrip_hive_parquet_spark_to_dask(spark_session, tmpdir, engine):
     tmpdir = str(tmpdir)
 
@@ -103,8 +114,7 @@ def test_roundtrip_hive_parquet_spark_to_dask(spark_session, tmpdir, engine):
     assert_eq(ddf, pdf.sort_index(axis=1), check_index=False)
 
 
-@pytest.mark.parametrize("npartitions", (1, 5, 10))
-@pytest.mark.parametrize("engine", ("pyarrow", "fastparquet"))
+@pytest.mark.parametrize("npartitions", [1, 5, 10])
 def test_roundtrip_parquet_dask_to_spark(spark_session, npartitions, tmpdir, engine):
     tmpdir = str(tmpdir)
     ddf = dd.from_pandas(pdf, npartitions=npartitions)
@@ -163,7 +173,6 @@ def test_roundtrip_parquet_spark_to_dask_extension_dtypes(spark_session, tmpdir)
     assert_eq(ddf, pdf, check_index=False)
 
 
-@pytest.mark.skipif(not PANDAS_GE_150, reason="Requires pyarrow-backed nullable dtypes")
 def test_read_decimal_dtype_pyarrow(spark_session, tmpdir):
     tmpdir = str(tmpdir)
     npartitions = 3
