@@ -43,7 +43,6 @@ from dask.utils import (
     random_state_data,
     skip_doctest,
     stringify,
-    stringify_collection_keys,
     takes_multiple_arguments,
     tmpfile,
     typename,
@@ -600,7 +599,7 @@ def test_derived_from():
     assert "not supported" in b_arg.lower()
     assert "dask" in b_arg.lower()
 
-    assert "  extra docstring\n\n" in Zap.f.__doc__
+    assert "extra docstring\n\n" in Zap.f.__doc__
 
 
 @pytest.mark.parametrize(
@@ -645,7 +644,10 @@ def test_derived_from_func():
 
 
 def test_derived_from_dask_dataframe():
+    dd = pytest.importorskip("pandas")
     dd = pytest.importorskip("dask.dataframe")
+    if dd._dask_expr_enabled():
+        pytest.xfail("we don't have docs yet")
 
     assert "inconsistencies" in dd.DataFrame.dropna.__doc__
 
@@ -780,17 +782,6 @@ def test_stringify():
     assert dsk[("y", 1)][1][0] == "('z', 1)"
 
 
-def test_stringify_collection_keys():
-    obj = "Hello"
-    assert stringify_collection_keys(obj) is obj
-
-    obj = [("a", 0), (b"a", 0), (1, 1)]
-    res = stringify_collection_keys(obj)
-    assert res[0] == str(obj[0])
-    assert res[1] == str(obj[1])
-    assert res[2] == obj[2]
-
-
 @pytest.mark.parametrize(
     "n,expect",
     [
